Package: GEOquery
Type: Package
Title: Get data from NCBI Gene Expression Omnibus (GEO)
Version: 2.80.0
Date: 2025-09-03
Authors@R: 
  c(person(given = "Sean",
           family = "Davis",
           role = c("aut", "cre"),
           email = "seandavi@gmail.com",
           comment = c(ORCID = "0000-0002-8991-6458")))
BugReports: https://github.com/seandavi/GEOquery/issues/new
Depends: R (>= 4.1.0), methods, Biobase
Imports: readr (>= 1.3.1), xml2, dplyr, data.table, tidyr, magrittr,
        limma, curl, rentrez, R.utils, stringr, SummarizedExperiment,
        S4Vectors, rvest, httr2
Suggests: knitr, rmarkdown, BiocGenerics, testthat, covr, markdown,
        quarto, DropletUtils, SingleCellExperiment
VignetteBuilder: quarto
URL: https://github.com/seandavi/GEOquery,
        http://seandavi.github.io/GEOquery,
        http://seandavi.github.io/GEOquery/
biocViews: Microarray, DataImport, OneChannel, TwoChannel, SAGE
Description: The NCBI Gene Expression Omnibus (GEO) is a public repository of microarray data.  Given the rich and varied nature of this resource, it is only natural to want to apply BioConductor tools to these data.  GEOquery is the bridge between GEO and BioConductor.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/GEOquery
git_branch: RELEASE_3_23
git_last_commit: 8b846af
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:18:19 UTC; biocbuild
Author: Sean Davis [aut, cre] (ORCID: <https://orcid.org/0000-0002-8991-6458>)
Maintainer: Sean Davis <seandavi@gmail.com>

# Generated by roxygen2: do not edit by hand

export(GDS2MA)
export(GDS2eSet)
export(browseGEOAccession)
export(browseWebsiteRNASeqSearch)
export(getGEO)
export(getGEOSeriesFileListing)
export(getGEOSuppFileURL)
export(getGEOSuppFiles)
export(getGEOfile)
export(getGSEDataTables)
export(getRNASeqData)
export(getRNASeqQuantGenomeInfo)
export(hasRNASeqQuantifications)
export(parseGEO)
export(searchFieldsGEO)
export(searchGEO)
export(urlForAccession)
exportMethods(Accession)
exportMethods(Columns)
exportMethods(GPL)
exportMethods(GPLList)
exportMethods(GSMList)
exportMethods(Meta)
exportMethods(Table)
exportMethods(dataTable)
exportMethods(show)
import(SummarizedExperiment)
import(rentrez)
importClassesFrom(Biobase,ExpressionSet)
importClassesFrom(limma,MAList)
importFrom(data.table,fread)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,select)
importFrom(magrittr,"%>%")
importFrom(readr,problems)
importFrom(readr,read_lines)
importFrom(readr,read_tsv)
importFrom(tidyr,gather)
importFrom(tidyr,separate)
importFrom(tidyr,spread)
importFrom(utils,download.file)
importFrom(xml2,read_html)
importFrom(xml2,read_xml)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_text)

## Status

<!-- badges: start -->
[![R-CMD-check](https://github.com/seandavi/GEOquery/workflows/R-CMD-check/badge.svg)](https://github.com/seandavi/GEOquery/actions)
[![Downloads](https://bioconductor.org/shields/downloads/release/GEOquery.svg)](https://bioconductor.org/packages/GEOquery)
[![Years in Bioc](https://bioconductor.org/shields/years-in-bioc/GEOquery.svg)](https://bioconductor.org/packages/GEOquery)
[![R-CMD-check](https://github.com/seandavi/GEOquery/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/seandavi/GEOquery/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

## Installation

To install from Bioconductor, use the following code:

```
# if needed, install BiocManager
install.packages("BiocManager")

# install GEOquery
BiocManager::install("GEOquery")
```

To install directly from github:

```
BiocManager::install('seandavi/GEOquery')
```

## Usage

See the full vignette in [rmarkdown](https://github.com/seandavi/GEOquery/blob/master/vignettes/GEOquery.Rmd) or visit Bioconductor for details:

- [Release version](http://www.bioconductor.org/packages/release/bioc/html/GEOquery.html)
- [Devel version](http://www.bioconductor.org/packages/devel/bioc/html/GEOquery.html)

## How to contribute

Contributions to GEOquery development can be submitted as a [pull request](https://github.com/seandavi/GEOquery/pulls) or a [feature request issue](https://github.com/seandavi/GEOquery/issues). We recommend following the [Bioconductor coding standards](https://contributions.bioconductor.org/r-code.html) where possible.  

fileOpen <- function(fname, ...) {
    con <- NULL
    if (!file.exists(fname)) {
        stop(sprintf("File %s does not appear to exist.", fname))
    }
    if (length(grep("\\.gz$", fname, perl = TRUE)) > 0) {
        con <- gzfile(fname, open = "rt")
    } else {
        con <- file(fname, "r")
    }
    return(con)
}

.onLoad <- function(lib, pkg) {
    packageStartupMessage("Setting options('download.file.method.GEOquery'='auto')")
    options(download.file.method.GEOquery = "auto")
    packageStartupMessage("Setting options('GEOquery.inmemory.gpl'=FALSE)")
    options(GEOquery.inmemory.gpl = FALSE)
}

#' Download a file from GEO soft file to the local machine
#'
#' This function simply downloads a SOFT format file associated with the GEO
#' accession number given.
#'
#' This function downloads GEO SOFT files based on accession number.  It does
#' not do any parsing.  The first two arguments should be fairly
#' self-explanatory, but the last is based on the input to the acc.cgi url at
#' the geo website.  In the default 'full' mode, the entire SOFT format file is
#' downloaded.  Both 'brief' and 'quick' offer shortened versions of the files,
#' good for 'peeking' at the file before a big download on a slow connection.
#' Finally, 'data' downloads only the data table part of the SOFT file and is
#' good for downloading a simple EXCEL-like file for use with other programs (a
#' convenience).
#'
#' @param GEO Character string, the GEO accession for download (eg., GDS84,
#' GPL96, GSE2553, or GSM10)
#' @param destdir Directory in which to store the resulting downloaded file.
#' Defaults to tempdir()
#' @param AnnotGPL A boolean defaulting to FALSE as to whether or not to use
#' the Annotation GPL information.  These files are nice to use because they
#' contain up-to-date information remapped from Entrez Gene on a regular basis.
#' However, they do not exist for all GPLs; in general, they are only available
#' for GPLs referenced by a GDS
#' @param amount Amount of information to pull from GEO.  Only applies to GSE,
#' GPL, or GSM.  See details...
#' @return Invisibly returns the full path of the downloaded file.
#'
#' @importFrom utils download.file
#'
#'
#' @author Sean Davis
#' @seealso \code{\link{getGEO}}
#' @references http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi
#' @keywords IO
#' @examples
#'
#' # myfile <- getGEOfile('GDS10')
#'
#' @export
getGEOfile <- function(GEO, destdir = tempdir(), AnnotGPL = FALSE, amount = c(
                         "full",
                         "brief", "quick", "data"
                       )) {
  amount <- match.arg(amount)
  geotype <- toupper(substr(GEO, 1, 3))
  mode <- "wb"
  GEO <- toupper(GEO)
  stub <- gsub("\\d{1,3}$", "nnn", GEO, perl = TRUE)
  if (geotype == "GDS") {
    gdsurl <- "https://ftp.ncbi.nlm.nih.gov/geo/datasets/%s/%s/soft/%s"
    myurl <- sprintf(gdsurl, stub, GEO, paste0(GEO, ".soft.gz"))
    destfile <- file.path(destdir, paste0(GEO, ".soft.gz"))
  }
  if (geotype == "GSE" & amount == "full") {
    gseurl <- "https://ftp.ncbi.nlm.nih.gov/geo/series/%s/%s/soft/%s"
    myurl <- sprintf(gseurl, stub, GEO, paste0(GEO, "_family.soft.gz"))
    destfile <- file.path(destdir, paste(GEO, ".soft.gz", sep = ""))
  }
  if (geotype == "GSE" & amount != "full" & amount != "table") {
    gseurl <- "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi"
    myurl <- paste(gseurl, "?targ=self&acc=", GEO, "&form=text&view=", amount,
      sep = ""
    )
    destfile <- file.path(destdir, paste(GEO, ".soft", sep = ""))
    mode <- "w"
  }
  if (geotype == "GPL") {
    if (AnnotGPL) {
      gplurl <- "https://ftp.ncbi.nlm.nih.gov/geo/platforms/%s/%s/annot/%s"
      myurl <- sprintf(gplurl, stub, GEO, paste0(GEO, ".annot.gz"))
      destfile <- file.path(destdir, paste(GEO, ".annot.gz", sep = ""))
      # check to see if Annotation GPL is present.  If so, use it, else
      # move on to submitter GPL
      res <- try(
        {
          if (!file.exists(destfile)) {
            downloadFile(myurl, destfile, mode)
          } else {
            message(sprintf(
              "Using locally cached version of %s found here:\n%s ",
              GEO, destfile
            ))
          }
        },
        silent = TRUE
      )
      if (!inherits(res, "try-error")) {
        return(invisible(destfile))
      } else {
        message("Annotation GPL not available, so will use submitter GPL instead")
      }
    }
    gseurl <- "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi"
    myurl <- paste(gseurl, "?targ=self&acc=", GEO, "&form=text&view=", amount,
      sep = ""
    )
    destfile <- file.path(destdir, paste(GEO, ".soft.gz", sep = ""))
    mode <- "w"
    if (!file.exists(destfile)) {
      downloadFile(myurl, destfile, mode = mode, quiet = TRUE)
    } else {
      message(sprintf(
        "Using locally cached version of %s found here:\n%s ",
        GEO, destfile
      ))
    }
    return(invisible(destfile))
  }
  if (geotype == "GSM") {
    gseurl <- "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi"
    myurl <- paste(gseurl, "?targ=self&acc=", GEO, "&form=text&view=", amount,
      sep = ""
    )
    destfile <- file.path(destdir, paste(GEO, ".soft", sep = ""))
    mode <- "w"
  }
  if (!file.exists(destfile)) {
    downloadFile(myurl, destfile, mode)
  } else {
    message(sprintf(
      "Using locally cached version of %s found here:\n%s ", GEO,
      destfile
    ))
  }
  invisible(destfile)
}

getGEORaw <- function(GEO, destdir = tempdir()) {
  return(getGEOSuppFiles(GEO, baseDir = destdir))
}


# internal use only
downloadFile <- function(url, destfile, mode, quiet = TRUE) {
  timeout_seconds <- max(getOption("timeout"), 120)

  result <- tryCatch(
    {
      req <- httr2::request(url) %>%
        httr2::req_headers(`accept-encoding` = "gzip") %>%
        httr2::req_timeout(timeout_seconds)

      resp <- httr2::req_perform(req)

      if (httr2::resp_status(resp) == 200) {
        writeBin(httr2::resp_body_raw(resp), destfile)
        return(TRUE)
      } else {
        stop("Failed to download file: ", httr2::resp_status(resp))
      }
    },
    error = function(e) {
      message(e)
      return(FALSE)
    }
  )

  message("File stored at:")
  message(destfile)

  ## if the download failed, remove the corrupted file and report the error
  if (!result) {
    if (file.exists(destfile)) {
      file.remove(destfile)
    }
    stop(sprintf("Failed to download %s!", destfile))
  }
  return(0)
}

#' @importClassesFrom limma MAList
#' @export
"GDS2MA" <- function(GDS, do.log2 = FALSE, GPL = NULL, AnnotGPL = TRUE, getGPL = TRUE) {
    if (!is.null(GPL) & getGPL) {
        GPL <- getGEO(Meta(GDS)$platform, AnnotGPL = AnnotGPL)
        ord.table <- match(Table(GDS)[, 1], Table(GPL)[, 1])
        # exclude non-numeric columns
    }
    inc.columns <- grep("GSM", colnames(Table(GDS)))
    mat <- suppressWarnings(as.matrix(apply(Table(GDS)[, inc.columns], 2, function(x) {
        as.numeric(as.character(x))
    })))
    if (do.log2) {
        M <- log2(mat)
    } else {
        M <- mat
    }
    if (is.null(GPL)) {
        ## No GPL requested
        return(new("MAList", list(M = M, A = NULL, targets = Columns(GDS), notes = Meta(GDS))))
    } else {
        ## include GPL
        return(new("MAList", list(M = M, A = NULL, targets = Columns(GDS), genes = Table(GPL)[ord.table,
            ], notes = Meta(GDS))))
    }

    return(MA)
}

#' @importClassesFrom Biobase ExpressionSet
#' @export
"GDS2eSet" <- function(GDS, do.log2 = FALSE, GPL = NULL, AnnotGPL = TRUE, getGPL = TRUE) {
    # exclude non-numeric columns
    inc.columns <- grep("GSM", colnames(Table(GDS)))
    mat <- suppressWarnings(as.matrix(apply(Table(GDS)[, inc.columns], 2, function(x) {
        as.numeric(as.character(x))
    })))
    if (do.log2) {
        expr <- log2(mat)
    } else {
        expr <- mat
    }
    rownames(expr) <- as.character(Table(GDS)$ID_REF)
    tmp <- Columns(GDS)
    rownames(tmp) <- as.character(tmp$sample)
    pheno <- new("AnnotatedDataFrame", data = tmp)
    mabstract = ifelse(is.null(Meta(GDS)$description), "", Meta(GDS)$description)
    mpubmedids = ifelse(is.null(Meta(GDS)$pubmed_id), "", Meta(GDS)$pubmed_id)
    mtitle = ifelse(is.null(Meta(GDS)$title), "", Meta(GDS)$title)
    featuredata = new("AnnotatedDataFrame", data = data.frame(row.names = rownames(expr)))
    if (getGPL | !is.null(GPL)) {
        if (is.null(GPL)) {
            GPL <- getGEO(Meta(GDS)$platform, AnnotGPL = AnnotGPL)
        }
        ord.table <- match(Table(GDS)[, 1], Table(GPL)[, 1])
        dt <- Table(GPL)
        rownames(dt) <- as.character(dt$ID)
        vardt = data.frame(Column = Columns(GPL)[, 1], labelDescription = Columns(GPL)[,
            2])
        ## GEO started using the same column names for both GO IDs and textual
        ## descriptions, so had to be made unique.
        vardt[, 1] <- make.unique(as.character(vardt[, 1]))
        rownames(vardt) = vardt[, 1]
        colnames(dt) <- rownames(vardt)
        featuredata <- new("AnnotatedDataFrame", data = dt[ord.table, ], varMetadata = vardt)
    }
    eset <- new("ExpressionSet", exprs = expr, phenoData = pheno, featureData = featuredata,
        experimentData = new("MIAME", abstract = mabstract, title = mtitle, pubMedIds = mpubmedids,
            other = Meta(GDS)))
    return(eset)
}

library(testthat)
library(limma)
context('GEO conversions')

test_that("GDS2eSet works", {
    gds = getGEO(filename=system.file("extdata/GDS507.soft.gz",package="GEOquery"))

    eset = GDS2eSet(gds)

    expect_is(eset,'ExpressionSet') # eset should be an ExpressionSet
    expect_equivalent(pubMedIds(experimentData(eset)),'14641932') #basic experimentData check failed
    expect_equivalent(nrow(eset),22645) #'eset has wrong number of rows')
    expect_equivalent(ncol(eset),17) #'eset has wrong number of columns')
})
    
test_that("GDS2MA works", {
    gds = getGEO(filename=system.file("extdata/GDS507.soft.gz",package="GEOquery"))
    
    malist = GDS2MA(gds)

    expect_is(malist,'MAList') #,'malist should be an MAList')
    expect_equivalent(nrow(malist),22645) #'malist has wrong number of rows')
    expect_equivalent(ncol(malist),17) #malist has wrong number of columns')
})

library(GEOquery)
context('get records without GPL')

test_that("GSE without GPL works", {
    gse = getGEO('GSE2553',getGPL=FALSE)[[1]]
    
    expect_true(validObject(gse))
    expect_equivalent(0,ncol(fData(gse)))
    expect_equivalent(12600,nrow(fData(gse)))
})


test_that("GDS without GPL works", {
    gds = getGEO('GDS10')
    
    eset = GDS2eSet(gds,getGPL=FALSE)
    expect_true(validObject(eset))
    expect_equivalent(0,ncol(fData(eset)))
    expect_equivalent(39114,nrow(fData(eset)))
})

#' Get GSE data tables from GEO into R data structures.
#' 
#' In some cases, instead of individual sample records (GSM) containing
#' information regarding sample phenotypes, the GEO Series contains that
#' information in an attached data table.  And example is given by GSE3494
#' where there are two data tables with important information contained within
#' them.  Using getGEO with the standard parameters downloads the GSEMatrix
#' file which, unfortunately, does not contain the information in the data
#' tables.  This function simply downloads the ``header'' information from the
#' GSE record and parses out the data tables into R data.frames.
#' 
#' 
#' @param GSE The GSE identifier, such as ``GSE3494''.
#' @return A list of data.frames.
#' @author Sean Davis <sdavis2@@mail.nih.gov>
#' @seealso \code{\link{getGEO}}
#'
#' @importFrom xml2 xml_text xml_find_all read_xml
#' @importFrom readr read_tsv
#' 
#' @keywords IO
#' @examples
#' 
#' dfl = getGSEDataTables('GSE3494')
#' lapply(dfl,head)
#'
#' 
#' @export 
getGSEDataTables <- function(GSE) {
    url = sprintf("https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?targ=self&form=xml&view=full&acc=%s",
        GSE)
    doc1 = read_xml(url)
    dTableNodes = xml_find_all(doc1, "//d1:Data-Table")
    dTables = sapply(dTableNodes, function(x) {
        cnames = sapply(xml_find_all(x, "d1:Column/d1:Name"), xml_text)
        dTableText = xml_text(xml_find_all(x, "d1:Internal-Data")[[1]])
        dTable = suppressWarnings(read_tsv(dTableText, col_names = FALSE))
        colnames(dTable) = cnames
        return(dTable)
    })
    return(dTables)
}

require(rentrez)
require(purrr)
require(lubridate)

.docSumListConvert = function(x) {
    ret = data.frame(
        uid          = x %>% purrr::map('uid') %>% purrr::flatten_chr() %>% as.integer(),
        accession    = x %>% purrr::map('accession') %>% purrr::flatten_chr(),
        seriestitle  = x %>% purrr::map('seriestitle') %>% purrr::flatten_chr(),
        platformtitle= x %>% purrr::map('platformtitle') %>% purrr::flatten_chr(),
        taxon        = x %>% purrr::map('gpl') %>% purrr::map(strsplit,';') %>% purrr::flatten() %>% I(),
        entrytype    = x %>% purrr::map('entrytype') %>% purrr::flatten_chr(),
        gdstype      = x %>% purrr::map('gdstype') %>% purrr::flatten_chr(),
        ptechtype    = x %>% purrr::map('ptechtype') %>% purrr::flatten_chr(),
        valtype      = x %>% purrr::map('valtype') %>% purrr::flatten_chr(),
        ssinfo       = x %>% purrr::map('ssinfo') %>% purrr::map(strsplit,';') %>% purrr::flatten() %>% I(),
        title        = x %>% purrr::map('title') %>% purrr::flatten_chr(),
        summary      = x %>% purrr::map('summary') %>% purrr::flatten_chr(),
        gpl          = x %>% purrr::map('gpl') %>% purrr::map(strsplit,';') %>% purrr::flatten() %>% purrr::map(function(x) paste0('GPL',x)) %>% I(),
        gse          = x %>% purrr::map('gse') %>% purrr::map(strsplit,';') %>% purrr::flatten() %>% purrr::map(function(x) paste0('GSE',x)) %>% I(),
        gds          = x %>% purrr::map('gds') %>% purrr::map(strsplit,';') %>% purrr::flatten() %>% purrr::map(function(x) ifelse(nchar(x)>0,paste0('GDS',x),"")) %>% I(),
        samples      = x %>% purrr::map('samples') %>% purrr::map('accession') %>% I(),
        suppfile     = x %>% purrr::map('suppfile') %>% purrr::map(strsplit,', ') %>% purrr::flatten() %>% I(),
        ftplink      = x %>% purrr::map('ftplink') %>% purrr::flatten_chr(),
        n_samples    = x %>% purrr::map('n_samples') %>% as.integer(),
        pubmedids    = x %>% purrr::map('pubmedids') %>% I(),
        projects     = x %>% purrr::map('projects') %>% I(),
        public_date  = x %>% purrr::map('pdat') %>% purrr::flatten_chr() %>% lubridate::date(),
        samplestaxa  = x %>% purrr::map('samplestaxa') %>% I() #purrr::map(strsplit,', ') %>% purrr::flatten() %>% I()
        )
    return(ret)
    }

.docSumListFix = function(x) {
  if(!is.list(x)) x = list(x)
  ret = lapply(x,function(val) {
    val$gpl = list(paste0('GPL',stringi::stri_split(val$gpl,fixed=';')[[1]]))
    val$gse = list(paste0('GSE',stringi::stri_split(val$gse,fixed=';')[[1]]))
    val$gds = list(paste0('GDS',stringi::stri_split(val$gds,fixed=';')[[1]]))
    val$ssinfo  = unique(stringi::stri_split(val$ssinfo,fixed=';')[[1]])
    return(val)
  })
  return(ret)
}

#' get all GEO records as docsums
#'
#' Fetches all records from NCBI entrez and creates a data frame
#' from the docsum entries, loadable into a SQL-like database
#'
#'
searchGEODocSums = function(term='GPL[ETYP] OR GSE[ETYP] OR GSM[ETYP] OR GDS[ETYP]') {
    res = rentrez::entrez_search(db='gds',use_history=TRUE,term=term)
    return(res)
}
fetchDocSums= function(res,retstart=1,retmax=100) {
  z=NULL
  while(is.null(z) || inherits(z,'error')) {
    z = try(rentrez::entrez_summary(db='gds',
                                    web_history=res$web_history,
                                    retstart=retstart,
                                    retmax=retmax))
  }
    
  return(.docSumListConvert(z))
}
.bqSchemaFromDocsums = function(exampleDF) {
  mappings = list(integer=list(mode='nullable',type='integer'),
                  factor =list(mode="nullable",type="string"),
                  character = list(mode='nullable',type='string'),
                  numeric = list(mode="nullable",type="float"),
                  logical = list(mode="nullable",type="boolean"),
                  AsIs    = list(mode="repeated",type="string"),
                  Date    = list(mode="nullable",type="string"))
  coltypes = sapply(exampleDF,class)
  coldesc  = mappings[coltypes]
  names(coldesc)=NULL
  coldesc=lapply(seq_along(coldesc),function(col) {
    ret = coldesc[[col]]
    ret$name=colnames(exampleDF)[col]
    return(ret)
  })
  return(coldesc)
}


getGEOMeta = function(geo) {
    .getGSMmeta = function(x) {
        ret = list(
            accession     = xml_attr(xml_find_first(dat,'/d1:MINiML/d1:Sample'),'iid'),
            gpl           = xml_attr(xml_find_first(dat,'/d1:MINiML/d1:Platform'),'iid'),
            title         = xml_text(xml_find_first(dat,'/d1:MINiML/d1:Sample/d1:Title')),
            description   = xml_text(xml_find_first(dat,'/d1:MINiML/d1:Sample/d1:Description')),
            type          = xml_text(xml_find_first(dat,'/d1:MINiML/d1:Sample/d1:Type')),
            submission_date= xml_text(xml_find_first(dat,'/d1:MINiML/d1:Sample/d1:Status/d1:Submission-Date')),
            last_update   = xml_text(xml_find_first(dat,'/d1:MINiML/d1:Sample/d1:Status/d1:Last-Update--Date')),
            release_date  = xml_text(xml_find_first(dat,'/d1:MINiML/d1:Sample/d1:Status/d1:Release-Date')),
            n_channels    = as.integer(xml_text(xml_find_first(dat,'/d1:MINiML/d1:Sample/d1:Channel-Count'))),
            channels      = data.frame(do.call(rbind,lapply(xml_find_all(dat,'/d1:MINiML/d1:Sample/d1:Channel'),
                                   function(channel) {
                                       return(list(
                                           tax_id = xml_attr(xml_find_first(channel,'./d1:Organism'),'taxid'),
                                           organism = xml_text(xml_find_first(channel,'./d1:Organism')),
                                           source = xml_text(xml_find_first(channel,'./d1:Source')),
                                           molecule = xml_text(xml_find_first(channel,'./d1:Molecule')),
                                           treatment_protocol = xml_text(xml_find_first(channel,'./d1:Treatment-Protocol')),
                                           extract_protocol = str_trim(xml_text(xml_find_first(channel,'./d1:Extract-Protocol'))),
                                           label_protocol = xml_text(xml_find_first(channel,'./d1:Label-Protocol')),
                                           scan_protocol = xml_text(xml_find_first(channel,'./d1:Scan-Protocol')),
                                           hybridization_protocol = xml_text(xml_find_first(channel,'./d1:Hybridization-Protocol')),
                                           growth_protocol = str_trim(xml_text(xml_find_first(channel,'./d1:Growth-Protocol'))),
                                           characteristics = sapply(xml_find_all(channel,'d1:Characteristics'),function(x) {str_trim(xml_text(x))}) %>%
                                             setNames(sapply(xml_find_all(channel,'d1:Characteristics'),function(x) {str_trim(xml_attrs(x,'tag'))})) %>% I()
                                       ))
                                   }))),
            columns       = data.frame(do.call(rbind,lapply(xml_find_all(dat,'/d1:MINiML/d1:Sample/d1:Data-Table/d1:Column'),
                                function(column) {
                                  return(list(
                                    name = xml_text(xml_find_first(column,'d1:Name')),
                                    description = xml_text(xml_find_first(column,'d1:Description'))
                                    
                                  ))
                                })))
                               
        )
        return(ret)
    }
    geo = toupper(geo)
    stopifnot(substr(geo,1,3) %in% c('GSE','GSM','GPL','GDS'))
              
    dat = read_xml(sprintf("https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=%s&view=full&targ=self&form=xml",geo))
    ret = switch(substr(geo,1,3),
                 GSM = .getGSMmeta(dat),
                 NA)
    return(ret)
}

#' Generic functions for GEOquery
#' 
#' The main documentation is in the Class documentation
#' 
#' @name GEOData-accessors
#' @aliases dataTable Accession Columns GPLList GSMList Meta Table
#' @author Sean Davis
#' @seealso \code{\link{GEOData-class}}
#' @keywords IO
NULL





#' Class 'GDS'
#' 
#' A class describing a GEO GDS entity
#' 
#' 
#' @name GDS-class
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new('GDS', ...)}
#' @author Sean Davis
#' @seealso \code{\link{GEOData-class}}
#' @keywords classes
NULL



#' Convert a GDS data structure to a BioConductor data structure
#' 
#' Functions to take a GDS data structure from getGEO and coerce it to limma
#' MALists or ExpressionSets.
#' 
#' This function just rearranges one data structure into another.  For GDS, it
#' also deals appropriately with making the 'targets' list item for the limma
#' data structure and the phenoData slot of ExpressionSets.
#'
#' @name coercion
#' 
#' @aliases GDS2MA GDS2eSet
#' @param GDS The GDS datastructure returned by getGEO
#' @param do.log2 Boolean, should the data in the GDS be log2 transformed
#' before inserting into the new data structure
#' @param GPL Either a GPL data structure (from a call to getGEO) or NULL.  If
#' NULL, this will cause a call to getGEO to produce a GPL.  The gene
#' information from the GPL is then used to construct the \code{genes} slot of
#' the resulting limma \code{MAList} object or the \code{featureData} slot of
#' the \code{ExpressionSet} instance.
#' @param AnnotGPL In general, the annotation GPL files will be available for
#' GDS records, so the default is to use these files over the user-submitted
#' GPL files
#' @param getGPL A boolean defaulting to TRUE as to whether or not to download
#' and include GPL information when converting to ExpressionSet or MAList.  You
#' may want to set this to FALSE if you know that you are going to annotate
#' your featureData using Bioconductor tools rather than relying on information
#' provided through NCBI GEO.  Download times can also be greatly reduced by
#' specifying FALSE.
#' @return \item{GDS2MA}{A limma MAList} \item{GDS2eSet}{An ExpressionSet
#' object}
#' @author Sean Davis
#' @references See the limma and ExpressionSet help in the appropriate packages
#' @keywords IO
#' @examples
#' 
#' 
#' \dontrun{gds505 <- getGEO('GDS505')}
#' \dontrun{MA <- GDS2MA(gds505)}
#' \dontrun{eset <- GDS2eSet(gds505)}
#' 
#' 
NULL





#' Class 'GEOData'
#' 
#' A virtual class for holding GEO samples, platforms, and datasets
#' 
#' 
#' @name GEOData-class
#' @aliases GEOData-class Accession,GEOData-method Columns,GEOData-method
#' Meta,GEOData-method Table,GEOData-method dataTable,GEOData-method
#' show,GEOData-method
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new('GEOData', ...)}.
#' @author Sean Davis
#' @seealso \code{\link{GDS-class}}, \code{\link{GPL-class}},
#' \code{\link{GSM-class}}, \code{\link{GEODataTable-class}},
#' @keywords classes
NULL





#' Class 'GEODataTable'
#' 
#' Contains the column descriptions and data for the datatable part of a GEO
#' object
#' 
#' 
#' @name GEODataTable-class
#' @aliases GEODataTable-class Accession,GEODataTable-method
#' Columns,GEODataTable-method Meta,GEODataTable-method
#' Table,GEODataTable-method dataTable,GEODataTable-method
#' show,GEODataTable-method
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new('GEODataTable', ...)}.
#' @author Sean Davis
#' @keywords classes
NULL





#' Class 'GPL'
#' 
#' Contains a full GEO Platform entity
#' 
#' @aliases GPL GPL,GDS-method
#' @name GPL-class
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new('GPL', ...)}.
#' @author Sean Davis
#' @seealso \code{\link{GEOData-class}}
#' @keywords classes
NULL





#' Class 'GSE'
#' 
#' Contains a GEO Series entity
#' 
#' 
#' @name GSE-class
#' @aliases GSE-class GPLList,GSE-method GSMList,GSE-method Meta,GSE-method
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new('GSE', ...)}.
#' @author Sean Davis
#' @seealso \code{\link{GPL-class}},\code{\link{GSM-class}}
#' @keywords classes
NULL





#' Class 'GSM'
#' 
#' A class containing a GEO Sample entity
#' 
#' 
#' @name GSM-class
#' @docType class
#' @section Objects from the Class: Objects can be created by calls of the form
#' \code{new('GSM', ...)}.
#' @author Sean Davis
#' @seealso \code{\link{GEOData-class}}
#' @keywords classes
NULL

#' Get a GEO object from NCBI or file
#' 
#' This function is the main user-level function in the GEOquery package.  It
#' directs the download (if no filename is specified) and parsing of a GEO SOFT
#' format file into an R data structure specifically designed to make access to
#' each of the important parts of the GEO SOFT format easily accessible.
#' 
#' getGEO functions to download and parse information available from NCBI GEO
#' (\url{http://www.ncbi.nlm.nih.gov/geo}).  Here are some details about what
#' is avaible from GEO.  All entity types are handled by getGEO and essentially
#' any information in the GEO SOFT format is reflected in the resulting data
#' structure.
#' 
#' From the GEO website:
#' 
#' The Gene Expression Omnibus (GEO) from NCBI serves as a public repository
#' for a wide range of high-throughput experimental data. These data include
#' single and dual channel microarray-based experiments measuring mRNA, genomic
#' DNA, and protein abundance, as well as non-array techniques such as serial
#' analysis of gene expression (SAGE), and mass spectrometry proteomic data. At
#' the most basic level of organization of GEO, there are three entity types
#' that may be supplied by users: Platforms, Samples, and Series.
#' Additionally, there is a curated entity called a GEO dataset.
#' 
#' A Platform record describes the list of elements on the array (e.g., cDNAs,
#' oligonucleotide probesets, ORFs, antibodies) or the list of elements that
#' may be detected and quantified in that experiment (e.g., SAGE tags,
#' peptides). Each Platform record is assigned a unique and stable GEO
#' accession number (GPLxxx). A Platform may reference many Samples that have
#' been submitted by multiple submitters.
#' 
#' A Sample record describes the conditions under which an individual Sample
#' was handled, the manipulations it underwent, and the abundance measurement
#' of each element derived from it. Each Sample record is assigned a unique and
#' stable GEO accession number (GSMxxx). A Sample entity must reference only
#' one Platform and may be included in multiple Series.
#' 
#' A Series record defines a set of related Samples considered to be part of a
#' group, how the Samples are related, and if and how they are ordered. A
#' Series provides a focal point and description of the experiment as a whole.
#' Series records may also contain tables describing extracted data, summary
#' conclusions, or analyses. Each Series record is assigned a unique and stable
#' GEO accession number (GSExxx).
#' 
#' GEO DataSets (GDSxxx) are curated sets of GEO Sample data. A GDS record
#' represents a collection of biologically and statistically comparable GEO
#' Samples and forms the basis of GEO's suite of data display and analysis
#' tools. Samples within a GDS refer to the same Platform, that is, they share
#' a common set of probe elements. Value measurements for each Sample within a
#' GDS are assumed to be calculated in an equivalent manner, that is,
#' considerations such as background processing and normalization are
#' consistent across the dataset. Information reflecting experimental design is
#' provided through GDS subsets.
#' 
#' @param GEO A character string representing a GEO object for download and
#' parsing.  (eg., 'GDS505','GSE2','GSM2','GPL96')
#' @param filename The filename of a previously downloaded GEO SOFT format file
#' or its gzipped representation (in which case the filename must end in .gz).
#' Either one of GEO or filename may be specified, not both.  GEO series matrix
#' files are also handled.  Note that since a single file is being parsed, the
#' return value is not a list of esets, but a single eset when GSE matrix files
#' are parsed.
#' @param destdir The destination directory for any downloads.  Defaults to the
#' architecture-dependent tempdir.  You may want to specify a different
#' directory if you want to save the file for later use.  Doing so is a good
#' idea if you have a slow connection, as some of the GEO files are HUGE!
#' @param GSElimits This argument can be used to load only a contiguous subset
#' of the GSMs from a GSE.  It should be specified as a vector of length 2
#' specifying the start and end (inclusive) GSMs to load.  This could be useful
#' for splitting up large GSEs into more manageable parts, for example.
#' @param GSEMatrix A boolean telling GEOquery whether or not to use GSE Series
#' Matrix files from GEO.  The parsing of these files can be many
#' orders-of-magnitude faster than parsing the GSE SOFT format files.  Defaults
#' to TRUE, meaning that the SOFT format parsing will not occur; set to FALSE
#' if you for some reason need other columns from the GSE records.
#' @param AnnotGPL A boolean defaulting to FALSE as to whether or not to use
#' the Annotation GPL information.  These files are nice to use because they
#' contain up-to-date information remapped from Entrez Gene on a regular basis.
#' However, they do not exist for all GPLs; in general, they are only available
#' for GPLs referenced by a GDS
#' @param getGPL A boolean defaulting to TRUE as to whether or not to download
#' and include GPL information when getting a GSEMatrix file.  You may want to
#' set this to FALSE if you know that you are going to annotate your
#' featureData using Bioconductor tools rather than relying on information
#' provided through NCBI GEO.  Download times can also be greatly reduced by
#' specifying FALSE.
#' @param parseCharacteristics A boolean defaulting to TRUE as to whether or not
#' to parse the characteristics information (if available) for a GSE Matrix file.
#' Set this to FALSE if you experience trouble while parsing the characteristics.
#' @return An object of the appropriate class (GDS, GPL, GSM, or GSE) is
#' returned.  If the GSEMatrix option is used, then a list of ExpressionSet
#' objects is returned, one for each SeriesMatrix file associated with the GSE
#' accesion.  If the filename argument is used in combination with a GSEMatrix
#' file, then the return value is a single ExpressionSet.
#' @section Warning : Some of the files that are downloaded, particularly those
#' associated with GSE entries from GEO are absolutely ENORMOUS and parsing
#' them can take quite some time and memory.  So, particularly when working
#' with large GSE entries, expect that you may need a good chunk of memory and
#' that coffee may be involved when parsing....
#' 
#' @importFrom readr problems
#' 
#' @author Sean Davis
#' @seealso \code{\link{getGEOfile}}
#' @keywords IO
#' @examples
#' 
#' gds <- getGEO("GDS10")
#' gds
#'
#' gse <- getGEO('GSE10')
#' # Returns a list, so look at first item
#' 
#' gse[[1]]
#' 
#' @export
getGEO <- function(GEO=NULL,
                   filename=NULL,
                   destdir=tempdir(),
                   GSElimits=NULL,GSEMatrix=TRUE,
                   AnnotGPL=FALSE,
                   getGPL=TRUE,
                   parseCharacteristics=TRUE) {
  con <- NULL
  if(!is.null(GSElimits)) {
    if(length(GSElimits)!=2) {
      stop('GSElimits should be an integer vector of length 2, like (1,10) to include GSMs 1 through 10')
    }
  }
  if(is.null(GEO) & is.null(filename)) {
    stop("You must supply either a filename of a GEO file or a GEO accession")
  }
  if(is.null(filename)) {
    GEO <- toupper(GEO)
    geotype <- toupper(substr(GEO,1,3))
    if(GSEMatrix & geotype=='GSE') {
      return(getAndParseGSEMatrices(GEO,destdir,AnnotGPL=AnnotGPL,getGPL=getGPL,parseCharacteristics=parseCharacteristics))
    }
    filename <- getGEOfile(GEO,destdir=destdir,AnnotGPL=AnnotGPL)
  }      
  ret <- parseGEO(filename,GSElimits,destdir,AnnotGPL=AnnotGPL,getGPL=getGPL)
  return(ret)
}       

library(GEOquery)
context('GPL')

test_that("generic GPL parsing works as expected", {
    gpl = getGEO('GPL96')
    
    expect_is(gpl,'GPL') #gpl is not a GPL object!')
    expect_is(gpl,'GEOData') #gpl does not inherit from GEOData!')
    expect_equivalent(nrow(Table(gpl)),22283) # nrow(gpl) does not match!')
    expect_equivalent(ncol(Table(gpl)),16) #ncol(gpl) does not match!')
    expect_is(Meta(gpl), 'list')  #Meta(gpl) should be a list')
    expect_is(Table(gpl), 'data.frame') #Table(gpl) should be a data.frame')
    expect_is(dataTable(gpl), 'GEODataTable') #dataTable(gpl) should be a GEODataTable')
})

test_that("quoted GPL works", {
    gpl = getGEO('GPL4133')

    expect_equivalent(45220,nrow(Table(gpl))) #GPL4133 should have 45220 rows
})

test_that("short GPL works", {
    gpl = getGEO('GPL15505')
    
    expect_is(gpl,'GPL')
    expect_equivalent(52,nrow(Table(gpl))) #GPL15505 should have 52 rows
})

test_that("GPL with no data table works", {
    gpl = getGEO("GPL5082")
    
    expect_is(gpl,'GPL')
    expect_equivalent(0,nrow(Table(gpl))) #GPL5082 has no data table!
})

library(GEOquery)
context("GDS")

test_that("generic GDS parsing works as expected", {
  gds <- getGEO(filename = system.file(
    "extdata/GDS507.soft.gz",
    package = "GEOquery"
  ))

  expect_is(gds, "GDS") # gds should be a "GDS"
  expect_is(Meta(gds), "list") # Meta(gds) should be a list
  expect_is(Table(gds), "data.frame") # Table(gds) should be a data.frame
  expect_is(
    dataTable(gds), "GEODataTable"
  ) # dataTable(gds) should be a GEODataTable
  expect_equivalent(nrow(Table(gds)), 22645) # gds has wrong number of rows!
  expect_equivalent(ncol(Table(gds)), 19) # gds has wrong number of columns!
  expect_length(Meta(gds), 23) # gds has wrong number of Metadata entries!
})

library(testthat)
context('GSM')

test_that("basic GSM works", {
    gsm = getGEO('GSM11805')

    expect_is(gsm,'GSM')
    expect_is(Meta(gsm),'list')
    expect_is(Table(gsm),'data.frame')
    expect_is(dataTable(gsm),'GEODataTable')
    expect_equivalent(Accession(gsm),'GSM11805')
    expect_equivalent(nrow(Table(gsm)),22283)
    expect_equivalent(ncol(Table(gsm)),3)
    expect_length(Meta(gsm),28)
})

library(GEOquery)
context("GSE")


test_that("empty GSE is handled correctly", {
    gse = getGEO('GSE11413')

    expect_is(gse, 'list')
    expect_is(gse[[1]], 'ExpressionSet')
    expect_equal(nrow(pData(gse[[1]])),12)
    expect_equal(nrow(fData(gse[[1]])),0)
})

test_that("case-mismatched IDs in GSEs handled correctly", {
    gse = getGEO('GSE35683')

    expect_equivalent(nrow(gse[[1]]), 54675)
})

test_that("single-sample GSE handled correctly", {
    gse = getGEO("GSE11595")
    
    expect_is(gse[[1]], "ExpressionSet")
    expect_equivalent(ncol(gse[[1]]),1)
})

test_that("short GSE handled correctly", {
    gse = getGEO("GSE34145")

    expect_equivalent(nrow(gse[[1]]), 15)
})

test_that("generic SOFT format GSE handled correctly", {
    gse = getGEO('GSE1563',GSEMatrix=FALSE)

    expect_equal(62, length(GSMList(gse)))
    expect_equal(1, length(GPLList(gse)))
    expect_equal(12625, nrow(Table(GPLList(gse)[[1]])))
    expect_equal(12625, nrow(Table(GSMList(gse)[[1]])))
    sapply(GSMList(gse),function(x) {
        expect_is(x,'GSM') # all GSMList members should be "GSM"s
        expect_equivalent(12625, nrow(Table(x)))}) # all GSMs should have the 12625 rows
    expect_is(GPLList(gse)[[1]],'GPL')
})

test_that("GSE with more than one value per characteristic handled", {
  gse = getGEO("GSE71989")
  
  expect_equivalent(nrow(gse[[1]]), 54675)
  expect_equivalent(ncol(gse[[1]]), 22)
})


test_that("GSE has populated experimentData", {
  gse = getGEO("GSE53986")
  
  ed <- experimentData(gse[[1]])
  expect_equal(pubMedIds(ed), "24739962")
  
  ei <- expinfo(ed)
  expect_equivalent(ei[1], "Jason,A,Hackney")
  expect_equivalent(ei[2], "") #lab
  expect_equivalent(ei[3], "hackney.jason@gene.com")
  expect_equivalent(ei[4], "NRROS negatively regulates ROS in phagocytes during host defense and autoimmunity")
  expect_equivalent(ei[5], "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE53986") #url
})

test_that("GSE populates experimentData as much as possible", {
  gse = getGEO("GSE99709")
  
  ed <- experimentData(gse[[1]])
  expect_equal(pubMedIds(ed), "")
  
  ei <- expinfo(ed)
  expect_equivalent(ei[1], "John,,Mariani")
  expect_equivalent(ei[2], "") #lab
  expect_equivalent(ei[3], "john_mariani@urmc.rochester.edu")
  expect_equivalent(ei[4], "RNA-Sequencing of Stat3 silenced oligodendrocyte progenitor cells.")
  expect_equivalent(ei[5], "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE99709") #url
  # ----------------------------------------------------------------
  gse = getGEO("GSE27712")
  
  ed <- experimentData(gse[[1]])
  expect_equal(pubMedIds(ed), "22253802")
   
  ei <- expinfo(ed)
  expect_equivalent(ei[1], "Joachim,L,Schultze")
  expect_equivalent(ei[2], "") #lab
  expect_equivalent(ei[3], "j.schultze@uni-bonn.de")
  expect_equivalent(ei[4], "GC424 tumor cells and gastric tumors")
  expect_equivalent(ei[5], "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE27712") #url
  expect_equivalent(abstract(ed), "This SuperSeries is composed of the SubSeries listed below.")
})

test_that("Empty files produces an error", {
  suppressWarnings(
    expect_error(getGEO(filename = system.file('extdata/GPLbroken.soft.gz', package="GEOquery")))
  )
})

test_that("GSE/GPL with integer64 columns handled correctly", {
  gse = getGEO("GSE7864")[[1]]
  fdata = fData(gse)
  expect_s3_class(fdata$ID, "integer64")
  expect_is(rownames(fdata), "character")
})

test_that("Test regression against issue 144", {
  gse = getGEO("GSE225759")[[1]]
  expect_equivalent(nrow(gse), 442)
  expect_equivalent(ncol(gse), 272)
})

test_that("GSE425 parsing with malformed sample lines", {
  gse = getGEO("GSE425")
  expect_is(gse, 'list')
  expect_is(gse[[1]], 'ExpressionSet')
  expect_true(ncol(gse[[1]]) > 0)
  expect_true(nrow(pData(gse[[1]])) > 0)
})

#' Search GEO database
#'
#' This function searches the [GDS](https://www.ncbi.nlm.nih.gov/gds)
#' database, and return a data.frame for all the search results.
#'
#' The NCBI allows users to access more records (10 per second) if they register
#' for and use an API key. [set_entrez_key][rentrez::set_entrez_key] function
#' allows users to set this key for all calls to rentrez functions during a
#' particular R session. You can also set an environment variable `ENTREZ_KEY`
#' by [Sys.setenv][base::Sys.setenv].  Once this value is set to your key
#' rentrez will use it for all requests to the NCBI. Details see
#' <https://docs.ropensci.org/rentrez/articles/rentrez_tutorial.html#rate-limiting-and-api-keys>
#'
#' @param query character, the search term. The NCBI uses a search term syntax
#' which can be associated with a specific search field with square brackets.
#' So, for instance "Homo sapiens\[ORGN\]" denotes a search for `Homo sapiens`
#' in the “Organism” field. Details see
#' <https://www.ncbi.nlm.nih.gov/geo/info/qqtutorial.html>. The names and
#' definitions of these fields can be identified using
#' [searchFieldsGEO].
#'
#' @seealso [searchFieldsGEO]
#'
#' @param step the number of records to fetch from the database each time. You
#' may choose a smaller value if failed.
#'
#' @return a data.frame contains the search results
#'
#' @examples
#' \dontrun{
#' searchGEO("diabetes[ALL] AND Homo sapiens[ORGN] AND GSE[ETYP]")
#' }
#'
#' @export
searchGEO <- function(query, step = 500L) {
  records_num <- rentrez::entrez_search(
    "gds", query,
    retmax = 0L
  )$count
  seq_starts <- seq(1L, records_num, step)
  records <- character(length(seq_starts))
  search_res <- rentrez::entrez_search(
    "gds", query,
    use_history = TRUE, retmax = 0L
  )
  for (i in seq_along(seq_starts)) {
    records[[i]] <- rentrez::entrez_fetch(
      db = "gds", web_history = search_res$web_history,
      rettype = "summary", retmode = "text",
      retmax = step, retstart = seq_starts[[i]]
    )
    Sys.sleep(1L)
  }
  records <- strsplit(
    gsub("^\\n|\\n$", "", paste0(records, collapse = "")),
    "\\n\\n"
  )[[1L]]
  name_value_pairs <- parse_name_value_pairs(preprocess_records(records))
  data.table::setDF(name_value_pairs)
  name_value_pairs
}


#' Provide a list of possible search fields for GEO search
#'
#' @import rentrez
#'
#' @returns a data.frame with names of possible search fields for GEO search
#' as well as descriptions, data types, etc. for each field. Fields are
#' in rows and their properties are in columns.
#'
#' @seealso \code{\link{searchGEO}}
#'
#' @examples
#' searchFieldsGEO()
#'
#' @export
searchFieldsGEO <- function() {
  res <- do.call(
    rbind,
    rentrez::entrez_db_searchable("gds")
  ) |> data.frame()
  rownames(res) <- NULL
  res
}



# this function just processed GEO searched results returned by `entrez_fetch`
# into key-values paris
preprocess_records <- function(x) {
  x <- sub("^\\d+\\.", "Title:", x, perl = TRUE)
  x <- sub(
    "\\n\\(Submitter supplied\\)\\s*",
    "\nSummary: ", x,
    perl = TRUE
  )
  x <- gsub(
    "\\s*\\n?(Platform|Dataset)s?\\s*:\\s*",
    "\n\\1s: ", x,
    perl = TRUE
  )
  x <- sub("\\tID:\\s*", "\nID: ", x, perl = TRUE)
  x <- sub(
    "\\n?\\s*((?:\\s*\\d+(?:\\s*related)?\\s*(?:DataSet|Platform|Sample|Serie)s?)+)([^:])",
    "\nContains: \\1\\2", x,
    perl = TRUE
  )
  x <- gsub(":\\t+", ": ", x, perl = TRUE)
  x <- gsub("\\t\\t+", " ", x, perl = TRUE)
  strsplit(x, "\\n", perl = TRUE)
}

# parse key-value pairs separeted by ":". For a list of key-value pairs
# characters (like: `list(c("a:1", "b:2"), c("a:3", "b:4"))`), this function
# simply cleans those up and transforms the list into a list object, the names
# of returned value is the unique keys in the pairs, the element of the returned
# list is the values in the paris.
# See parse_name_value_pairs(list(c("a:1", "b:2"), c("a:3", "b:4")))
#' @return a list, every element of which corresponds to each key-value pairs
#' group by key in the paris.
#' @noRd
parse_name_value_pairs <- function(chr_list, sep = ":") {
  .characteristic_list <- lapply(chr_list, function(x) {
    if (!length(x)) {
      return(data.table::data.table())
    }
    # Don't use `data.table::tstrsplit`, as it will split string into three
    # or more elements.
    name_value_pairs <- data.table::transpose(
      str_split(x, paste0("(\\s*+)", sep, "(\\s*+)"))
    )
    res <- as.list(name_value_pairs[[2L]])
    names(res) <- name_value_pairs[[1L]]
    data.table::setDT(res)
    res
  })
  characteristic_dt <- data.table::rbindlist(
    .characteristic_list,
    use.names = TRUE, fill = TRUE
  )
  data.table::setnames(characteristic_dt, make.unique)

  # parse text into corresponding atomic vector mode
  lapply(characteristic_dt, function(x) {
    data.table::fread(
      text = x, sep = "", header = FALSE,
      strip.white = TRUE, blank.lines.skip = FALSE, fill = TRUE
    )[[1L]]
  })
}

# split string based on pattern, Only split once, Return a list of character,
# the length of every element is two
str_split <- function(string, pattern, ignore.case = FALSE) {
  regmatches(
    string,
    regexpr(pattern, string,
      perl = TRUE, fixed = FALSE,
      ignore.case = ignore.case
    ),
    invert = TRUE
  )
}

test_that("returns_correct_structure_when_geo_accessible", {
  result <- searchFieldsGEO()

  expect_s3_class(result, "data.frame")
  expect_true(
    all(c("Name", "Description", "FullName") %in%
      colnames(result))
  )
})

.na_strings = c("NA", "null", "NULL", "Null")


#' @importFrom data.table fread
.read_lines <- function(...) {
    data.table::fread(..., sep = "", header = FALSE)[[1]]
}

#' Parse GEO text
#' 
#' Workhorse GEO parsers.
#' 
#' These are probably not useful to the end-user.  Use getGEO to access these
#' functions.  parseGEO simply delegates to the appropriate specific parser.
#' There should be no reason to use the parseGPL, parseGDS, parseGSE, or
#' parseGSM functions directly.
#' 
#' @aliases parseGEO parseGPL parseGSE parseGDS parseGSM
#' @param fname The filename of a SOFT format file.  If the filename ends in
#' .gz, a gzfile() connection is used to read the file directly.
#' @param GSElimits Used to limit the number of GSMs parsed into the GSE
#' object; useful for memory management for large GSEs.
#' @param destdir The destination directory into which files will be saved (to
#' be used for caching)
#' @param AnnotGPL Fetch the annotation GPL if available
#' @param getGPL Fetch the GPL associated with a GSEMatrix entity (should
#' remain TRUE for all normal use cases)
#' @return parseGEO returns an object of the associated type.  For example, if
#' it is passed the text from a GDS entry, a GDS object is returned.
#' @author Sean Davis
#' @seealso \code{\link{getGEO}}
#' @keywords IO
#' 
#' @export
parseGEO <- function(fname, GSElimits, destdir = tempdir(), AnnotGPL = FALSE, getGPL = TRUE) {
    con <- fileOpen(fname)
    first.entity <- findFirstEntity(con)
    close(con)
    ret <- switch(as.character(first.entity[1]), sample = {
        parseGSM(fname)
    }, series = parseGSE(fname, GSElimits), dataset = {
        parseGDS(fname)
    }, platform = {
        parseGPL(fname)
    }, `0` = {
        parseGSEMatrix(fname, destdir = destdir, AnnotGPL = AnnotGPL, getGPL = getGPL)$eset
    }, )
    return(ret)
}

parseGeoMeta <- function(txt) {
    ## leader <- strsplit(grep('!\\w*_',txt,perl=TRUE,value=TRUE)[1],'_')[[1]][1]
    ## pull out only lines that are in the header
    tmp <- txt[grep("!\\w*?_", txt)]
    tmp <- gsub("!\\w*?_", "", tmp)
    first.eq <- regexpr(" = ", tmp)
    tmp <- cbind(substring(tmp, first = 1, last = first.eq - 1), substring(tmp, first = first.eq +
        3))
    tmp <- tmp[tmp[, 1] != "", ]
    header <- split(tmp[, 2], tmp[, 1])
    return(header)
}

parseGDSSubsets <- function(txt) {
    ## takes GDS text as input returns a data frame suitable for inclusion as a
    ## 'Column' slot in a GDS GeoDataTable object
    numSubsets <- length(grep("^\\^subset", txt, ignore.case = TRUE))
    subset.lists <- list()
    if (numSubsets > 0) {
        subset.types <- do.call("rbind", strsplit(txt[grep("!subset_type", txt)], " = "))[,
            2]
        subset.descriptions <- do.call("rbind", strsplit(txt[grep("!subset_description",
            txt)], " = "))[, 2]
        subset.samples <- strsplit(do.call("rbind", strsplit(txt[grep("!subset_sample_id",
            txt)], " = "))[, 2], ",")
        for (i in 1:length(subset.types)) {
            for (j in 1:length(subset.samples[[i]])) {
                subset.lists[[subset.types[i]]][[subset.samples[[i]][j]]] <- subset.descriptions[i]
            }
        }
    }
    sample.descriptions <- do.call("rbind", strsplit(txt[grep("#GSM", txt)], " = "))
    sample.descriptions[, 1] <- gsub("#", "", sample.descriptions[, 1])
    samp.table <- data.frame(sample = as.character(sample.descriptions[, 1]))
    if (length(subset.lists) > 0) {
        for (i in 1:length(subset.lists)) {
            samp.table[match(names(subset.lists[[i]]), samp.table[, 1]), names(subset.lists)[i]] <- as.factor(unlist(subset.lists[[i]]))
        }
        colnames(samp.table) <- c("sample", gsub(" ", ".", names(subset.lists)))
    } else {
        colnames(samp.table) <- "sample"
    }
    samp.table[match(sample.descriptions[, 1], samp.table[, 1]), "description"] <- sample.descriptions[,
        2]
    return(as.data.frame(samp.table))
}


splitOnFirst <- function(x, pattern) {
    patlen <- nchar(pattern)
    matches <- regexpr(pattern, x)
    leftside <- substr(x, start = 1, stop = matches - 1)
    rightside <- substr(x, start = matches + patlen, stop = 1e+07)
    return(data.frame(leftside, rightside))
}


parseGeoColumns <- function(txt) {
    cols <- as.data.frame(splitOnFirst(txt[grep("^#", txt, perl = TRUE)], " = "))
    cols[, 1] <- sub("#", "", as.character(cols[, 1]))
    colnames(cols) <- c("Column", "Description")
    return(cols)
}

### Parse a GSM; the limit is established by the parameter n, used by getGSE to
### limit the number of lines read to only the size of ONE GSM, since a single
### GSE contains many GSMs
#' @importFrom readr read_lines
.parseGSMWithLimits <- function(con, n = NULL) {
    txt <- vector("character")
    i <- 0
    hasDataTable = FALSE
    while (i <- i + 1) {
        tmp <- try(read_lines(con, 1))
        if (inherits(tmp, "try-error") | length(tmp) == 0) {
            i <- i - 1  # counted an extra line
            hasDataTable = FALSE
            break
        }
        txt[i] <- tmp
        if (i == length(txt))
            txt <- c(txt, character(i))  # double vector size
        if (length(grep("!\\w+_table_begin", txt[i], perl = TRUE)) > 0) {
            hasDataTable = TRUE
            break
        }
    }
    txt <- txt[1:i]
    cols <- parseGeoColumns(txt)
    meta <- parseGeoMeta(txt)
    geoDataTable <- new("GEODataTable", columns = data.frame(), table = data.frame())
    if (hasDataTable) {
        nLinesToRead <- NULL
        if (!is.null(n)) {
            nLinesToRead <- n - length(txt)
        }
        dat3 <- fastTabRead(con, n = nLinesToRead)
        geoDataTable <- new("GEODataTable", columns = cols, table = dat3[1:(nrow(dat3) -
            1), ])
    }
    gsm <- new("GSM", header = meta, dataTable = geoDataTable)
}


### This function does a grep on a file by doing a readline in chunks of size
### chunksize.  Return value is a data.frame with the line number of each match
### and the line itself.
filegrep <- function(con, regex, chunksize = 10000) {
    i <- 0
    ret <- NULL
    while (TRUE) {
        lines <- readLines(con, n = chunksize)
        if (length(lines) == 0) {
            break
        }
        foundLines <- grep(regex, lines)
        foundTypes <- lines[foundLines]
        if (length(foundLines) > 0) {
            foundLines <- foundLines + i
            tmp <- data.frame(foundLines = foundLines, foundTypes = foundTypes)
            if (is.null(ret)) {
                ret <- tmp
            } else {
                ret <- rbind(ret, tmp)
            }
        }
        i <- i + length(lines)
    }
    return(ret)
}

#' @importFrom readr read_lines
parseGSE <- function(fname, GSElimits = NULL) {
    gsmlist <- list()
    gpllist <- list()
    GSMcount <- 0
    message("Reading file....")
    lines = .read_lines(fname)
    message("Parsing....")
    lineCounts <- grep("\\^(SAMPLE|PLATFORM)", lines)
    message(sprintf("Found %d entities...", length(lineCounts)))
    ## I close and reopen the file because on Windows, the seek function is
    ## pretty much guaranteed to NOT work This gets the header information for
    ## the GSE
    a <- lines[1:(lineCounts[1] - 1)]
    header = parseGeoMeta(a)
    lineCounts = c(lineCounts, length(lines))
    ## parse the actual entities, now
    for (j in 1:(length(lineCounts) - 1)) {
        tmp <- strsplit(lines[lineCounts[j]], " = ")[[1]]
        accession <- tmp[2]
        message(sprintf("%s (%d of %d entities)", accession, j, length(lineCounts)))
        entityType <- tolower(sub("\\^", "", tmp[1]))
        if (entityType == "sample") {
            gsmlist[[accession]] <- .parseGSMTxt(lines[lineCounts[j]:(lineCounts[j +
                1] - 1)])
        }
        if (entityType == "platform") {
            gpllist[[accession]] <- .parseGPLTxt(lines[lineCounts[j]:(lineCounts[j +
                1] - 1)])
        }
    }
    return(new("GSE", header = header, gsms = gsmlist, gpls = gpllist))
}


findFirstEntity <- function(con) {
    while (TRUE) {
        line <- suppressWarnings(readLines(con, 100))
        if (length(line) == 0)
            return(0)
        entity.line <- grep("^\\^(DATASET|SAMPLE|SERIES|PLATFORM|ANNOTATION)", line,
            ignore.case = TRUE, value = TRUE, perl = TRUE)
        entity.line <- gsub("annotation", "platform", entity.line, ignore.case = TRUE)
        if (length(entity.line) > 0) {
            ret <- c(tolower(sub("\\^", "", strsplit(entity.line, " = ")[[1]][1])),
                strsplit(entity.line, " = ")[[1]][2])
            return(ret)
        }
        # catch GSE SeriesMatrix files
        checkline = grep("^!Series_title", line, ignore.case = TRUE, value = TRUE,
            perl = TRUE)
        # Messy, but GSEMatrix files use tab-separation rather than '=' for
        # separation, so if the ' = ' is not present, we presume to have a
        # GSEMatrix file (return 0)
        if (length(checkline) > 0) {
            if (!grepl(" = ", checkline))
                return(0)
        }
    }
    return(0)
}

fastTabRead <- function(con, sep = "\t", header = TRUE, sampleRows = 100, colClasses = NULL,
    n = NULL, quote = "\"", ...) {
    ### Need to read tables quickly, so guess the colclasses on the fly.  This is
    ### a bit dangerous since the first rows might not be representative of the
    ### entire set, but it is SO MUCH FASTER than the alternative, I have to do
    ### it.
    dat3 <- data.frame(NULL)
    numberOfLines <- -1
    if (!is.null(n)) {
        numberOfLines <- n - sampleRows
    }
    if (is.null(colClasses)) {
        if (!is.null(n)) {
            sampleRows <- min(sampleRows, n)
        }
        dat1 <- read.table(con, sep = sep, header = header, nrows = sampleRows, fill = TRUE,
            check.names = FALSE, quote = quote, comment.char = "", na.strings = c("NA",
                "null", "NULL", "Null"), ...)
        colclasses <- apply(dat1, 2, class)
        colclasses[1] <- "factor"
        dat2 <- try(read.delim(con, sep = sep, colClasses = colclasses, header = FALSE,
            quote = quote, comment.char = "", na.strings = c("NA", "null", "NULL",
                "Null"), nrows = numberOfLines, ...), silent = TRUE)
        if (inherits(dat2, "try-error")) {
            dat3 = dat1
        } else {
            colnames(dat2) <- colnames(dat1)
            dat3 <- rbind(dat1, dat2)
        }
    } else {
        dat3 <- read.delim(con, sep = sep, colClasses = colClasses, header = header,
            quote = quote, comment.char = "", na.strings = c("NA", "null", "NULL",
                "Null"), nrows = numberOfLines, ...)
    }
    return(dat3)
}

.genericGEOTableParser <- function(txt) {
    # Find first line of the table
    has_table = TRUE
    tbl_begin = grep("!\\w+_table_begin", txt, perl = TRUE)[1] + 1
    # No data table at all, so everything is header info
    if (is.na(tbl_begin)) {
        tbl_begin = length(txt)
        has_table = FALSE
    }

    if (has_table) {
        metadata_rows <- seq_len(tbl_begin - 2)
    } else {
        metadata_rows <- seq_along(txt)
    }


    # Find last line of the table Edge case is that the record does not have a
    # table end line, in which case we use all lines
    tbl_end = grep("!\\w+_table_end", txt, perl = TRUE)
    if (length(tbl_end) == 0) {
        tbl_end = length(txt)
    } else {
        tbl_end = tbl_end[1] - 1
    }
    dat3 = data.frame()
    if (has_table) {
        dat3 <- data.table::fread(text = txt[tbl_begin:tbl_end], na = .na_strings)
    }
    return(list(meta_text = txt[metadata_rows], data_frame = dat3))
}

#' @importFrom data.table fread
parseGDS <- function(fname) {
    txt = data.table::fread(fname, sep = "")[[1]]

    parser_results = .genericGEOTableParser(txt)

    cols <- parseGDSSubsets(parser_results$meta_text)
    meta <- parseGeoMeta(parser_results$meta_text)
    geoDataTable <- new("GEODataTable", columns = cols, table = parser_results$data_frame)
    gds <- new("GDS", header = meta, dataTable = geoDataTable)
    return(gds)
}

.parseGPLWithLimits <- function(con, n = NULL) {
    txt <- vector("character")
    i <- 0
    hasDataTable = FALSE
    while (i <- i + 1) {
        tmp <- try(readLines(con, 1))
        if (inherits(tmp, "try-error") || length(tmp) == 0 || (!is.null(n) && i ==
            n)) {
            i <- i - 1  # counted an extra line
            hasDataTable = FALSE
            break
        }
        txt[i] <- tmp
        if (i == length(txt))
            txt <- c(txt, character(i))  # double vector size
        if (length(grep("!\\w+_table_begin", txt[i], perl = TRUE)) > 0) {
            hasDataTable = TRUE
            break
        }
    }
    txt <- txt[1:i]
    cols <- parseGeoColumns(txt)
    meta <- parseGeoMeta(txt)
    geoDataTable <- new("GEODataTable", columns = data.frame(), table = data.frame())
    if (hasDataTable) {
        nLinesToRead <- NULL
        if (!is.null(n)) {
            nLinesToRead <- n - length(txt)
        }
        dat3 <- fastTabRead(con, n = nLinesToRead, quote = "")
        geoDataTable <- new("GEODataTable", columns = cols, table = dat3[1:(nrow(dat3) -
            1), ])
    }
    gpl <- new("GPL", header = meta, dataTable = geoDataTable)
}


#' @importFrom readr read_tsv
.parseGSMTxt <- function(txt) {
    parser_results = .genericGEOTableParser(txt)

    cols <- parseGeoColumns(parser_results$meta_text)
    meta <- parseGeoMeta(parser_results$meta_text)
    geoDataTable <- new("GEODataTable", columns = cols, table = parser_results$data_frame)
    geo <- new("GSM", header = meta, dataTable = geoDataTable)
    return(geo)
}


#' @importFrom data.table fread
parseGSM <- function(fname) {
    txt = data.table::fread(fname, sep = "")[[1]]
    # read_lines reads separate blank lines on windows, so remove them before
    # proceeding. NOT doing so results in the Table header being repeated as the
    # first line of the Table and test failures galore.
    txt = txt[txt != ""]
    return(.parseGSMTxt(txt))
}

### In memory cache for GPL objects parsed from locally cached versions of GPL
### SOFT files.  It is disabled by default with
### options('GEOquery.inmemory.gpl'=FALSE).
GPLcache <- new.env(parent = emptyenv())


#' @importFrom readr read_tsv
.parseGPLTxt <- function(txt) {

    parser_results = .genericGEOTableParser(txt)

    cols <- parseGeoColumns(parser_results$meta_text)
    meta <- parseGeoMeta(parser_results$meta_text)
    geoDataTable <- new("GEODataTable", columns = cols, table = parser_results$data_frame)
    geo <- new("GPL", header = meta, dataTable = geoDataTable)
    return(geo)
}


#' @importFrom readr read_lines
parseGPL <- function(fname) {
    if (getOption("GEOquery.inmemory.gpl")) {
        info <- file.info(fname, extra_cols = FALSE)
        cache <- get0(fname, envir = GPLcache, inherits = FALSE)
        ## Check if the locally cached version wasn't modified.
        if (!is.null(cache) && cache$info$mtime == info$mtime) {
            message("Using GPL object found in memory from locally cached version.")
            return(cache$gpl)
        }
    }
    txt = data.table::fread(fname, sep = "")[[1]]
    # read_lines reads separate blank lines on windows, so remove them before
    # proceeding. NOT doing so results in the Table header being repeated as the
    # first line of the Table and test failures galore.
    txt = txt[txt != ""]
    return(.parseGPLTxt(txt))
}


txtGrab <- function(regex, x) {
    x <- as.character(x)
    a <- regexpr(regex, x, perl = TRUE)
    return(substr(x, a, a + attr(a, "match.length") - 1))
}

### Function wrapper to get and parse ALL the GSEMatrix files associated with a
### GSE into a list of ExpressionSets
getAndParseGSEMatrices <- function(GEO, destdir, AnnotGPL, getGPL = TRUE, parseCharacteristics = TRUE) {
    GEO <- toupper(GEO)
    ## This stuff functions to get the listing of available files for a given GSE
    ## given that there may be many GSEMatrix files for a given GSE.
    stub = gsub("\\d{1,3}$", "nnn", GEO, perl = TRUE)
    gdsurl <- "https://ftp.ncbi.nlm.nih.gov/geo/series/%s/%s/matrix/"
    b = getDirListing(sprintf(gdsurl, stub, GEO))
    message(sprintf("Found %d file(s)", length(b)))
    ret <- list()
    ## Loop over the files, returning a list, one element for each file
    for (i in 1:length(b)) {
        message(b[i])
        destfile = file.path(destdir, b[i])
        if (file.exists(destfile)) {
            message(sprintf("Using locally cached version: %s", destfile))
        } else {
            url = sprintf("https://ftp.ncbi.nlm.nih.gov/geo/series/%s/%s/matrix/%s",
                stub, GEO, b[i])
            downloadFile(url, destfile = destfile, mode = "wb")
        }
        ret[[b[i]]] <- parseGSEMatrix(destfile, destdir = destdir, AnnotGPL = AnnotGPL,
            getGPL = getGPL)$eset
    }
    return(ret)
}

#' Parse a GSE mstrix file
#'
#' Not meant for user calling, but parses
#' a single GSEMatrix file.
#'
#' @importFrom dplyr select filter mutate mutate_all
#' @importFrom tidyr gather spread separate
#' @importFrom readr read_lines
#' @importClassesFrom Biobase ExpressionSet
#' @importFrom magrittr %>%
#'
#' @param fname filename
#' @param AnnotGPL set to TRUE to get the annotation GPL version
#' @param destdir the destdination directory for download
#' @param getGPL whether or not to get the GPL associated
#' @param parseCharacteristics Whether or not to do full 'characteristic' parsing
#' @keywords internal
#' 
parseGSEMatrix <- function(fname, AnnotGPL = FALSE, destdir = tempdir(), getGPL = TRUE,
    parseCharacteristics = TRUE) {
    # Convenient VERY fast line reader based on data.table See:
    # https://stackoverflow.com/a/32924981/459633 data read into single character
    # column, so subset to get just text.
    dat <- data.table::fread(fname, sep = "")[[1]]

    ## get the number of !Series lines for header reading
    series_header_row_count <- sum(grepl("^!Series_", dat))
    series_table_begin_line = grep("^!series_matrix_table_begin", dat)
    series_table_end_line = grep("^!series_matrix_table_end", dat)
    if (length(series_table_begin_line) != 1) {
        stop("parsing failed--expected only one '!series_data_table_begin'")
    }
    # Read the !Series_ and !Sample_ lines
    header <- data.table::fread(fname, header = FALSE, nrows = series_header_row_count)
    # Extract only the actual !Sample_ lines to avoid issues with malformed/empty lines
    sample_lines <- dat[grepl("^!Sample_", dat)]
    tmpdat <- data.table::fread(text = sample_lines, header = FALSE, sep = "\t")

    headertmp <- t(header)
    headerdata <- rbind(data.frame(), headertmp[-1, ])
    colnames(headerdata) <- sub("!Series_", "", as.character(header[[1]]))
    headerlist <- lapply(split.default(headerdata, names(headerdata)), function(x) {
        as.character(Reduce(function(a, b) {
            paste(a, b, sep = "\n")
        }, x))
    })

    link = "https://www.ncbi.nlm.nih.gov/geo/"
    if (!is.null(headerlist$web_link)) {
        link <- headerlist$web_link
    } else if (!is.null(headerlist$geo_accession)) {
        link <- paste(link, "query/acc.cgi?acc=", headerlist$geo_accession, sep = "")
    }

    ed <- new("MIAME", name = ifelse(is.null(headerlist$contact_name), "", headerlist$contact_name),
        title = headerlist$title, contact = ifelse(is.null(headerlist$contact_email),
            "", headerlist$contact_email), pubMedIds = ifelse(is.null(headerlist$pubmed_id),
            "", headerlist$pubmed_id), abstract = ifelse(is.null(headerlist$summary),
            "", headerlist$summary), url = link, other = headerlist)

    tmptmp <- t(tmpdat)
    sampledat <- rbind(data.frame(), tmptmp[-1, ])
    colnames(sampledat) <- make.unique(sub("!Sample_", "", as.character(tmpdat[[1]])))
    sampledat[["geo_accession"]] = as.character(sampledat[["geo_accession"]])
    rownames(sampledat) = sampledat[["geo_accession"]]
    ## Lots of GSEs now use 'characteristics_ch1' and 'characteristics_ch2' for
    ## key-value pairs of annotation. If that is the case, this simply cleans
    ## those up and transforms the keys to column names and the values to column
    ## values.
    if (length(grep("characteristics_ch", colnames(sampledat))) > 0 && parseCharacteristics) {
        pd = sampledat %>%
            dplyr::select(dplyr::contains("characteristics_ch")) %>%
            dplyr::mutate(accession = rownames(.)) %>%
            # these next two lines avoid warnings due to columns having different
            # factor levels (attributes).
        mutate_all(as.character) %>%
            tidyr::gather(characteristics, kvpair, -accession) %>%
            dplyr::filter(grepl(":", kvpair) & !is.na(kvpair))
        # Thx to Mike Smith (@grimbough) for this code sometimes the
        # 'characteristics_ch1' fields are empty and contain no key:value pairs.
        # spread() will fail when called on an empty data_frame.  We catch this
        # case and remove the 'charactics_ch1' column instead
        if (nrow(pd)) {
            pd = dplyr::mutate(pd, characteristics = ifelse(grepl("_ch2", characteristics),
                "ch2", "ch1")) %>%
                tidyr::separate(kvpair, into = c("k", "v"), sep = ":", fill = "right",
                  extra = "merge") %>%
                dplyr::mutate(k = paste(k, characteristics, sep = ":")) %>%
                dplyr::select(-characteristics) %>%
                dplyr::filter(!is.na(v)) %>%
                dplyr::group_by(accession, k) %>%
                dplyr::mutate(v = paste0(trimws(v), collapse = ";")) %>%
                unique() %>%
                tidyr::spread(k, v)
        } else {
            pd = pd %>%
                dplyr::select(accession)
        }
        ## dplyr::mutate(characteristics=ifelse(grepl('_ch2',characteristics),'ch2','ch1'))
        ## %>% dplyr::filter(grepl(':',kvpair)) %>% tidyr::separate(kvpair, into=
        ## c('k','v'), sep=':') if(nrow(pd)>0) { pd = pd %>% dplyr::mutate(k =
        ## paste(k,characteristics,sep=':')) %>% dplyr::select(-characteristics)
        ## %>% tidyr::spread(k,v)
        sampledat = sampledat %>%
            dplyr::left_join(pd, by = c(geo_accession = "accession"))
    }

    datamat = NULL
    if (series_table_end_line - series_table_begin_line == 2) {
        datamat = read.table(textConnection(dat[(series_table_begin_line + 1):(series_table_end_line -
            1)]), header = TRUE, sep = "\t")
    } else {
        datamat <- data.table::fread(text = dat[(series_table_begin_line + 1):(series_table_end_line -
            1)], quote = "\"", na.strings = c("NA", "null", "NULL", "Null"))
    }
    ## kip = series_table_begin_line) comment.char = '!series_matrix_table_end')
    tmprownames = as.character(datamat[[1]])
    # need the as.matrix for single-sample or empty GSE
    datamat <- as.matrix(datamat[!is.na(tmprownames), -1])
    rownames(datamat) <- tmprownames[!is.na(tmprownames)]
    datamat <- as.matrix(datamat)
    rownames(sampledat) <- colnames(datamat)
    GPL = as.character(sampledat[1, grep("^platform_id", colnames(sampledat), ignore.case = TRUE)])
    ## if getGPL is FALSE, skip this and featureData is then a data.frame with no
    ## columns
    fd = new("AnnotatedDataFrame", data = data.frame(row.names = rownames(datamat)))
    if (getGPL) {
        gpl <- getGEO(GPL, AnnotGPL = AnnotGPL, destdir = destdir)
        vmd <- Columns(gpl)
        dat <- Table(gpl)
        ## GEO uses 'TAG' instead of 'ID' for SAGE GSE/GPL entries, but it is
        ## apparently always the first column, so use dat[,1] instead of dat$ID
        ## The next line deals with the empty GSE
        tmpnames = character(0)
        if (ncol(dat) > 0) {
            tmpnames = as.character(dat[, 1])
        }
        ## Fixed bug caused by an ID being 'NA' in GSE15197, for example
        tmpnames[is.na(tmpnames)] = "NA"
        rownames(dat) <- make.unique(tmpnames)
        ## Apparently, NCBI GEO uses case-insensitive matching between platform
        ## IDs and series ID Refs ???
        dat <- dat[match(tolower(rownames(datamat)), tolower(rownames(dat))), ]
        # Fix possibility of duplicate column names in the GPL files; this is
        # prevalent in the Annotation GPLs
        rownames(vmd) <- make.unique(colnames(Table(gpl)))
        colnames(dat) <- rownames(vmd)
        fd <- new("AnnotatedDataFrame", data = dat, varMetadata = vmd)
    }
    if (is.null(nrow(datamat))) {
        ## fix empty GSE datamatrix samplename stuff above does not work with
        ## empty GSEs, so fix here, also
        tmpnames <- names(datamat)
        rownames(sampledat) <- tmpnames
        datamat = matrix(nrow = 0, ncol = nrow(sampledat))
        colnames(datamat) <- tmpnames
    } else {
        ## This looks like a dangerous operation but is needed to deal with the
        ## fact that NCBI GEO allows case-insensitive matching and we need to
        ## pick one.
        rownames(datamat) <- rownames(dat)
    }
    eset <- new("ExpressionSet", phenoData = as(sampledat, "AnnotatedDataFrame"), annotation = GPL,
        featureData = fd, experimentData = ed, exprs = as.matrix(datamat))
    return(list(GPL = as.character(sampledat[1, grep("platform_id", colnames(sampledat),
        ignore.case = TRUE)]), eset = eset))
}

# build geo docsum jsons
res = searchGEODocSums()

library(BiocParallel)
register(MulticoreParam())



docsumJSON = function(retstart,retmax) {
  require(jsonlite)
  dir.create(sprintf('~/docsums/docsums_%d',(retstart-1) %% 10000),showWarnings=FALSE,recursive = TRUE)
  f = gzfile(sprintf('~/docsums/docsums_%d/docsums_%d_%d.json.gz',(retstart-1) %% 10000,retstart,retstart+retmax),'wt')
  h = fetchDocSums(res,retstart=retstart,retmax=retmax)
  stream_out(h,f)
  message(retstart)
  close(f)
}

buildGEODocsums = function() {
  bplapply(seq(0,res$count,100),function(x) docsumJSON(x+1,100))
}

#' The URL for a GEO accession
#'
#' Sometimes, you just need the URL for a GEO accession. This function
#' returns the URL for a given GEO accession number that
#' can be used to access the GEO page for that accession.
#'
#' @param geo A GEO accession number
#'
#' @return A character vector with the URL for the GEO accession
#'
#' @seealso \code{\link{browseGEOAccession}}
#'
#' @examples
#'
#' urlForAccession("GSE262484")
#'
#' @export
urlForAccession <- function(geo) {
  paste0("https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=", geo)
}

#' Open the GEO page for a given accession
#'
#' Sometimes, you just need to see the GEO website page for a
#' GEO accession. This function opens the GEO page for a given
#' accession number in the default browser.
#'
#' @param geo A GEO accession number
#'
#' @seealso \code{\link{urlForAccession}}
#'
#' @examples
#' \dontrun{
#' browseGEOAccession("GSE262484")
#' }
#'
#' @export
browseGEOAccession <- function(geo) {
  browseURL(urlForAccession(geo))
}

context("browsing geo with accession number")

test_that("returns_correct_url_for_valid_geo_accession", {
  geo <- "GSE262484"
  expected_url <- "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE262484"
  result <- urlForAccession(geo)
  expect_equal(result, expected_url)
})

#' get a directory listing from NCBI GEO
#' 
#' This one makes some assumptions about the structure of the HTML response
#' returned.
#'
#' @param url A URL, assumed to return an NCBI-formatted
#'   index page
#' 
#' @importFrom xml2 read_html xml_text xml_find_all 
getDirListing <- function(url) {
    # Takes a URL and returns a character vector of filenames
    a <- xml2::read_html(url)
    fnames = grep("^G", xml_text(xml_find_all(a, "//a/@href")), value = TRUE)
    return(fnames)
}

#' Get GEO supplemental file URL for a given GEO accession
#'
#' @param GEO
#'
#' @examples
#' # an example of a GEO supplemental file URL
#' # with a set of single-cell RNA-seq data
#' url = getGEOSuppFileURL("GSE161228")
#' url
#' 
#' \dontrun{
#'   browseURL(url)
#' }
#' 
#' @export
getGEOSuppFileURL <- function(GEO) {
    stub = gsub("\\d{1,3}$", "nnn", GEO, perl = TRUE)
    geotype <- toupper(substr(GEO, 1, 3))
    if (geotype == "GSM") {
        url <- sprintf("https://ftp.ncbi.nlm.nih.gov/geo/samples/%s/%s/suppl/", stub,
            GEO)
    }
    if (geotype == "GSE") {
        url <- sprintf("https://ftp.ncbi.nlm.nih.gov/geo/series/%s/%s/suppl/", stub,
            GEO)
    }
    if (geotype == "GPL") {
        url <- sprintf("https://ftp.ncbi.nlm.nih.gov/geo/platform/%s/%s/suppl/", stub,
            GEO)
    }
    return(url)
}



#' Get Supplemental Files from GEO
#' 
#' NCBI GEO allows supplemental files to be attached to GEO Series (GSE), GEO
#' platforms (GPL), and GEO samples (GSM).  This function 'knows' how to get
#' these files based on the GEO accession.  No parsing of the downloaded files
#' is attempted, since the file format is not generally knowable by the
#' computer.
#' 
#' Again, just a note that the files are simply downloaded.
#' 
#' @param GEO A GEO accession number such as GPL1073 or GSM1137
#' @param makeDirectory Should a 'subdirectory' for the downloaded files be
#' created?  Default is TRUE.  If FALSE, the files will be downloaded directly
#' into the baseDir.
#' @param baseDir The base directory for the downloads.  Default is the current
#' working directory.
#' @param filter_regex A character(1) regular expression that will be
#'     used to filter the filenames from GEO to limit those files that
#'     will be downloaded. This is useful to limit to, for example,
#'     bed files only.
#' @param fetch_files logical(1). If TRUE, then actually download the
#'     files. If FALSE, just return the filenames that would have been
#'     downloaded. Useful for testing and getting a list of files
#'     without actual download.
#' @return If fetch_files=TRUE, a data frame is returned invisibly with rownames representing the
#' full path of the resulting downloaded files and the records in the
#' data.frame the output of file.info for each downloaded file.
#' If fetch_files=FALSE, a data.frame of URLs and filenames is returned.
#' @author Sean Davis <sdavis2@@mail.nih.gov>
#' @keywords IO database
#' @examples
#' 
#' a <- getGEOSuppFiles('GSM1137', fetch_files = FALSE)
#' a
#' 
#' # with a set of single-cell RNA-seq data
#' a <- getGEOSuppFiles('GSE161228', fetch_files = FALSE)
#' a
#' 
#' @export
getGEOSuppFiles <- function(
    GEO,
    makeDirectory = TRUE,
    baseDir = getwd(),
    fetch_files = TRUE,
    filter_regex = NULL
) {
    geotype <- toupper(substr(GEO, 1, 3))
    storedir <- baseDir
    fileinfo <- list()
    url <- getGEOSuppFileURL(GEO)
    fnames <- try(getDirListing(url), silent = TRUE)
    if (inherits(fnames, "try-error")) {
        message("No supplemental files found.")
        message("Check URL manually if in doubt")
        message(url)
        return(NULL)
    }
    if (makeDirectory) {
        suppressWarnings(dir.create(storedir <- file.path(baseDir, GEO)))
    }
    if (!is.null(filter_regex)) {
        fnames = fnames[grepl(filter_regex, fnames)]
    }
    if (fetch_files) {
        for (i in fnames) {
            destfile <- file.path(storedir, i)


 
                if (!file.exists(destfile)) {
                    httr2::request(base_url=url) |>
                      httr2::req_url_path_append(i) |>
                      httr2::req_url_query(tool="geoquery") |>
                      httr2::req_perform(path=destfile)
                    res <- 0
                } else {
                  message(sprintf("Using locally cached version of supplementary file(s) %s found here:\n%s ",
                    GEO, destfile))
                    res <- 0
                }
            fileinfo[[destfile]] <- file.info(destfile)
        }
        ret <- do.call(rbind, fileinfo)
        ret$fname <- fnames
        ret$destdir <- storedir
        ret$filepath <- file.path(storedir, fnames)
        ret$GEO <- GEO
        return(ret)
    } else {
        return(data.frame(fname = fnames, url = file.path(url, fnames)))
    }
}

#' GSE Supplemental file listing
#' 
#' The GEO Series records often have one or more supplemental files.
#' In most cases, those files are archived as '.tar' files, the contents
#' of which are only available in a file listing file not present on the
#' website for download.
#' 
#' This function reads that file listing file and returns the results
#' as a data.frame. 
#' 
#' @returns A data.frame with 5 columns. See example. 
#' 
#' @param GSE character(1) the GSE accession
#' 
#' @examples
#' getGEOSeriesFileListing('GSE288770')
#' 
#' @export
getGEOSeriesFileListing <- function(GSE) {
  url = getGEOSuppFileURL(GSE)
  ret <- readr::read_tsv(file.path(url,'filelist.txt'))
  ret |> 
    dplyr::rename_with(tolower) |>
    dplyr::rename('archive_or_file'='#archive/file')
}

library(testthat)
context('GEO supplementary files')

test_that("GSE Supplemental files downloading works", {
    res = getGEOSuppFiles('GSE1000')
    expect_equivalent(nrow(res), 1)
})

test_that("GSM Supplemental files downloading works", {
    res = getGEOSuppFiles('GSM15789')
    expect_equivalent(nrow(res), 1)
})

test_that("GSM Supplemental files downloading to baseDir works", {
    res = getGEOSuppFiles('GSM15789', baseDir=tempdir())
    expect_equivalent(nrow(res), 1)
})

test_that("GSE supplemental file no-download works", {
    res = getGEOSuppFiles('GSE63137', fetch_files = FALSE)
    expect_equivalent(nrow(res), 12)
    expect_equivalent(ncol(res), 2)
})

test_that("GSE Supplemental file filtering works", {
    res = getGEOSuppFiles('GSE63137', fetch_files = FALSE, filter_regex = 'txt.gz')
    expect_equivalent(ncol(res), 2)
    expect_equivalent(nrow(res), 4)
})

#' get all download links from a GEO accession
#'
#' This function gets all download links from a GEO accession number.
#'
#' @param gse GEO accession number
#'
#' @return A character vector with all download links
#'
#' @keywords internal
getGSEDownloadPageURLs <- function(gse) {
  url <- "https://ncbi.nlm.nih.gov/geo/download/"
  links <- httr2::request(url) |>
    httr2::req_timeout(15) |>
    httr2::req_retry(3) |>
    httr2::req_url_query(acc = gse) |>
    httr2::req_perform() |>
    httr2::resp_body_string() |>
    rvest::read_html() |>
    rvest::html_nodes("a") |>
    rvest::html_attr("href") |>
    stringr::str_replace("^/geo/", "https://www.ncbi.nlm.nih.gov/geo/") |>
    stringr::str_replace("^ftp://", "https://")
  class(links) <- c("geoDownloadLinks", class(links))
  return(links)
}


#' Get the link to the raw counts file from GEO
#'
#' This function extracts the link to the raw counts file from a
#' geoDownloadLinks object.
#'
#' @param links A geoDownloadLinks object
#'
#' @return A character vector with the link to the raw counts file
#'
#' @keywords internal
getRNAQuantRawCountsURL <- function(links) {
  if (!inherits(links, "geoDownloadLinks")) {
    stop("Input must be a geoDownloadLinks object")
  }
  link <- stringr::str_subset(links, "raw_counts")
  return(link)
}

#' Get the RNA-seq quantification annotation link
#'
#' This function extracts the link to the RNA-seq quantification annotation
#' file from a geoDownloadLinks object.
#'
#' @param links A geoDownloadLinks object
#'
#' @return A character vector with the link to the annotation file
#'
#' @keywords internal
getRNAQuantAnnotationURL <- function(links) {
  if (!inherits(links, "geoDownloadLinks")) {
    stop("Input must be a geoDownloadLinks object")
  }
  link <- stringr::str_subset(links, "annot.tsv.gz")
  return(link)
}

#' Extract filename from a GEO download URL
#'
#' This function extracts the filename from a GEO download URL.
#' The filename is expected to be a query parameter called "file".
#' If the query parameter is not found, the function returns NULL.
#'
#' The idea is to use this function to extract filenames that
#' contain important metadata from the GEO RNA-seq quantification.
#'
#' In particular, the filename is expected to contain the genome build
#' and species information that we can attach to the SummarizedExperiment.
#'
#' @param url A GEO download URL
#'
#' @return A character vector with the filename
#'
#' @keywords internal
extractFilenameFromDownloadURL <- function(url) {
  # get a query parameter called "file" and its value from
  # the URL

  # example URL: https://www.ncbi.nlm.nih.gov/geo/download/\
  # ?format=file&type=rnaseq_counts&file=Human.GRCh38.p13.annot.tsv.gz
  parsed <- httr2::url_parse(url)
  fname <- NULL
  if ("query" %in% names(parsed)) {
    if ("file" %in% names(parsed$query)) {
      fname <- parsed$query$file
    }
  }
  fname
}

#' Extract genome build and species from a GEO download URL
#'
#' This function extracts the genome build and species information
#' from a GEO download URL. The genome build and species information
#' is expected to be in the filename of the download URL.
#'
#' @param url A GEO annotation file download URL
#'
#' @return A character vector with the genome build and species information
#'
#' @keywords internal
urlExtractRNASeqQuantGenomeInfo <- function(url) {
  fname <- extractFilenameFromDownloadURL(url)
  if (is.null(fname)) {
    return(NULL)
  }
  splits <- stringr::str_split(fname, "\\.")[[1]]
  genome_build <- paste0(splits[2], ".", splits[3])
  species <- splits[1]
  return(c(genome_build = genome_build, species = species, fname = fname))
}

#' Extract genome build and species for GEO RNA-seq quantification
#'
#' This function extracts the genome build and species information
#' for a GEO RNA-seq quantification.
#'
#' @param gse GEO accession number
#'
#' @return A character vector with the genome build and species information
#'
#' @examples
#' getRNASeqQuantGenomeInfo("GSE164073")
#'
#' @export
getRNASeqQuantGenomeInfo <- function(gse) {
  links <- getGSEDownloadPageURLs(gse)
  annotation_link <- getRNAQuantAnnotationURL(links)
  metadata <- urlExtractRNASeqQuantGenomeInfo(annotation_link)
  return(metadata)
}


#' Read RNA-seq quantification annotation from GEO
#'
#' This function reads the annotation file from a GEO link. The annotation
#' file is expected to be a tab-separated file with the first column
#' containing the gene IDs and the remaining columns containing the
#' annotation information.
#'
#' @param link A link to the annotation file
#'
#' @return A data frame of annotation information with gene IDs as row names
#'
#' @keywords internal
readRNAQuantAnnotation <- function(link) {
  annotation <- as.data.frame(readr::read_tsv(link, show_col_types = FALSE))
  rownames(annotation) <- as.character(annotation$GeneID)
  return(annotation)
}


#' Read raw counts from GEO
#'
#' This function reads the raw counts from a GEO link. The raw counts
#' are expected to be in a tab-separated file with the first column
#' containing the gene IDs and the remaining columns containing the
#' raw counts.
#'
#' This function reads the raw counts and returns a matrix with the
#' gene IDs as the row names, ready for use in creating a
#' SummarizedExperiment.
#'
#' @param link A link to the raw counts file
#'
#' @return A matrix of raw counts with gene IDs as row names
#'
#' @keywords internal
readRNAQuantRawCounts <- function(link) {
  quants <- readr::read_tsv(link, show_col_types = FALSE)
  gene_ids <- as.character(quants$GeneID)
  quants <- as.matrix(quants[, -1])
  rownames(quants) <- gene_ids
  return(quants)
}


#' Get RNA-seq quantification and annotation from GEO
#'
#' This function downloads the raw counts and annotation files from GEO
#' for a given GEO accession number.
#'
#' @param gse GEO accession number
#'
#' @return A list with two elements: quants (a matrix of raw counts) and
#' annotation (a data frame of annotation information).
#'
#' @keywords internal
getRNASeqQuantResults <- function(gse) {
  links <- getGSEDownloadPageURLs(gse)
  raw_counts_link <- getRNAQuantRawCountsURL(links)
  if (length(raw_counts_link) == 0) {
    stop(
      "No raw counts file found.\n",
      "Navigate to: \n  https://ncbi.nlm.nih.gov/geo/download/?acc=",
      gse,
      "\nand check if the 'RNA-Seq raw counts' link is available."
    )
  }
  annotation_link <- getRNAQuantAnnotationURL(links)
  quants <- readRNAQuantRawCounts(raw_counts_link)
  annotation <- readRNAQuantAnnotation(annotation_link)
  return(list(quants = quants, annotation = annotation))
}

#' Browse GEO search website for RNA-seq datasets
#'
#' This function opens a browser window to the NCBI GEO website
#' with a search for RNA-seq datasets. It is included as a convenience
#' function to remind users of how to search for RNA-seq datasets using
#' the NCBI GEO website and an "rnaseq counts" filter.
#'
#' @examples
#' \dontrun{
#' browseWebsiteRNASeqSearch()
#' }
#' @export
browseWebsiteRNASeqSearch <- function() {
  browseURL(
    "https://ncbi.nlm.nih.gov/gds?term=%22rnaseq%20counts%22%5BFilter%5D"
  )
}


#' Does a GEO accession have RNA-seq quantifications?
#'
#' This function checks if a GEO accession number has RNA-seq quantifications
#' available. It does this by checking if the GEO accession number has a
#' "RNA-Seq raw counts" link available on the GEO download page.
#'
#' @param accession GEO accession number
#'
#' @return TRUE if the GEO accession number has RNA-seq quantifications
#' available, FALSE otherwise.
#'
#' @examples
#' hasRNASeqQuantifications("GSE164073")
#'
#' @export
hasRNASeqQuantifications <- function(accession) {
  links <- getGSEDownloadPageURLs(accession)
  raw_counts_link <- getRNAQuantRawCountsURL(links)
  if (length(raw_counts_link) > 0) {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

#' Get GEO RNA-seq quantifications as a SummarizedExperiment object
#'
#' For human and mouse GEO datasets, NCBI GEO attempts to process
#' the raw data and provide quantifications in the form of raw counts
#' and an annotation file. This function downloads the raw counts and
#' annotation files from GEO and merges that with the metadata from the GEO
#' object to create a SummarizedExperiment.
#'
#' @details
#' A major barrier to fully exploiting and reanalyzing the massive volumes
#' of public RNA-seq data archived by SRA is the cost and effort required to
#' consistently process raw RNA-seq reads into concise formats that summarize
#' the expression results. To help address this need, the NCBI SRA and GEO
#' teams have built a pipeline that precomputes RNA-seq gene expression counts
#' and delivers them as count matrices that may be incorporated into commonly
#' used differential expression analysis and visualization software.
#'
#' The pipeline processes RNA-seq data from SRA using the HISAT2 aligner and
#' and then generates gene expression counts using the featureCounts program.
#'
#' See the
#' [GEO documentation](https://ncbi.nlm.nih.gov/geo/info/rnaseqcounts.html)
#' for more details.
#'
#' @import SummarizedExperiment
#'
#'
#' @param accession GEO accession number
#'
#' @return A SummarizedExperiment object with the raw counts as the counts
#' assay, the annotation as the rowData, and the metadata from GEO as
#' the colData.
#'
#'
#' @examples
#' se <- getRNASeqData("GSE164073")
#' se
#'
#' @export
getRNASeqData <- function(accession) {
  quantifications <- getRNASeqQuantResults(accession)
  se <- as(
    getGEO(accession)[[1]],
    "SummarizedExperiment"
  )
  names_matching <- intersect(
    colnames(quantifications$quants),
    rownames(SummarizedExperiment::colData(se))
  )
  old_metadata <- S4Vectors::metadata(se)
  old_metadata$genomeInfo <- getRNASeqQuantGenomeInfo(accession)
  old_metadata$created_at <- Sys.time()
  new_se <- SummarizedExperiment::SummarizedExperiment(
    assays = list(counts = quantifications$quants[, names_matching]),
    rowData = quantifications$annotation,
    colData = SummarizedExperiment::colData(se)[names_matching, ],
    metadata = old_metadata
  )
  new_se
}

library(testthat)
context("geo_rnaseq")

test_that("getGSEDownloadPageURLs returns character vector", {
  gse <- "GSE83322"
  result <- getGSEDownloadPageURLs(gse)

  expect_type(result, "character")
  expect_true(length(result) > 0)
})

test_that("getRNAQuantRawCountsURL returns character vector", {
  gse <- "GSE83322"
  links <- getGSEDownloadPageURLs(gse)
  result <- getRNAQuantRawCountsURL(links)

  expect_type(result, "character")
  expect_length(result, 1)
})

test_that("getRNAQuantAnnotationURL returns character vector", {
  gse <- "GSE83322"
  links <- getGSEDownloadPageURLs(gse)
  result <- getRNAQuantAnnotationURL(links)

  expect_type(result, "character")
  expect_length(result, 1)
})

links <- getGSEDownloadPageURLs("GSE83322")

test_that("readRNAQuantRawCounts returns matrix", {
  link <- getRNAQuantRawCountsURL(links)
  result <- readRNAQuantRawCounts(link)

  expect_true(is.matrix(result))
  expect_true(nrow(result) > 0)
  expect_true(ncol(result) > 0)
})

test_that("readRNAQuantAnnotation returns data.frame", {
  link <- getRNAQuantAnnotationURL(links)
  result <- readRNAQuantAnnotation(link)

  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 0)
  expect_true(ncol(result) > 0)
})

test_that("getRNASeqQuantResults returns list", {
  gse <- "GSE83322"
  result <- getRNASeqQuantResults(gse)

  expect_type(result, "list")
  expect_length(result, 2)
  expect_true(is.matrix(result$quants))
  expect_true(is.data.frame(result$annotation))
})

test_that("getRNASeqData returns SummarizedExperiment", {
  gse <- "GSE83322"
  result <- getRNASeqData(gse)

  expect_s4_class(result, "SummarizedExperiment")
  expect_true(nrow(result) > 0)
  expect_true(ncol(result) > 0)
})

test_that("hasRNASeqQuantifications returns logical", {
  gse <- "GSE83322"
  result <- hasRNASeqQuantifications(gse)

  expect_type(result, "logical")
})

test_that("hasRNASeqQuantifications returns TRUE for GSE83322", {
  gse <- "GSE83322"
  result <- hasRNASeqQuantifications(gse)

  expect_true(result)
})

test_that("hasRNASeqQuantifications returns FALSE for GSE2553", {
  gse <- "GSE2553"
  result <- hasRNASeqQuantifications(gse)

  expect_false(result)
})

test_that("getRNASeqQuantGenomeInfo returns correct data", {
  genome_info <- getRNASeqQuantGenomeInfo("GSE83322")

  expect_length(genome_info, 3)
  expect_true(genome_info["species"] == "Human")
  expect_true(stringr::str_starts(genome_info["genome_build"], "GR"))
})

### $Id$

### Generic GEO classes:

setClass("GEODataTable", representation(columns = "data.frame", table = "data.frame"),
    prototype = list(columns = data.frame(matrix(nr = 0, nc = 0)), table = data.frame(matrix(nr = 0,
        nc = 0))))


setClass("GEOData", representation(header = "list"), prototype = list(header = list()))

setClass("GSE", representation(header = "list", gsms = "list", gpls = "list"), prototype = list(header = list(),
    gsms = list(), gpls = list()))

setClass("GPL", representation(dataTable = "GEODataTable"), prototype = list(dataTable = new("GEODataTable")),
    contains = "GEOData")
setClass("GSM", representation(dataTable = "GEODataTable"), prototype = list(dataTable = new("GEODataTable")),
    contains = "GEOData")
setClass("GDS", representation(gpl = "GPL", dataTable = "GEODataTable"), prototype = list(gpl = new("GPL"),
    dataTable = new("GEODataTable")), contains = "GEOData")

printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth printHead
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth <-
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth function(x)
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth #
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth Print
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth leading
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth 5
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth elements
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth or
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth rows
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth of
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth atomic
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth object
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth From
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth limma
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth and
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth Gordon
printHead <- function(x) #  Print leading 5 elements or rows of atomic object   From limma and Gordon Smyth Smyth
{
    if (is.atomic(x)) {
        d <- dim(x)
        if (length(d) < 2)
            which <- "OneD"
        if (length(d) == 2)
            which <- "TwoD"
        if (length(d) > 2)
            which <- "Array"
    } else {
        if (inherits(x, "data.frame")) {
            d <- dim(x)
            which <- "TwoD"
        } else which <- "Recursive"
    }
    switch(which, OneD = {
        n <- length(x)
        if (n > 20) {
            print(x[1:5])
            cat(n - 5, "more elements ...\n")
        } else print(x)
    }, TwoD = {
        n <- d[1]
        if (n > 10) {
            print(x[1:5, ])
            cat(n - 5, "more rows ...\n")
        } else print(x)
    }, Array = {
        n <- d[1]
        if (n > 10) {
            dn <- dimnames(x)
            dim(x) <- c(d[1], prod(d[-1]))
            x <- x[1:5, ]
            dim(x) <- c(5, d[-1])
            if (!is.null(dn[[1]])) dn[[1]] <- dn[[1]][1:5]
            dimnames(x) <- dn
            print(x)
            cat(n - 5, "more rows ...\n")
        } else print(x)
    }, Recursive = print(x))
}

chopColumns <- function(x) {
    apply(x, 2, substring, first = 1, last = 35)
}

setGeneric("Meta", function(object) standardGeneric("Meta"))
setGeneric("Accession", function(object) standardGeneric("Accession"))
setGeneric("dataTable", function(object) standardGeneric("dataTable"))
setGeneric("Columns", function(object) standardGeneric("Columns"))
setGeneric("GPL", function(object) standardGeneric("GPL"))
setGeneric("GSM", function(object) standardGeneric("GSM"))
setGeneric("GPLList", function(object) standardGeneric("GPLList"))
setGeneric("GSMList", function(object) standardGeneric("GSMList"))
setGeneric("Table", function(object) standardGeneric("Table"))

#' @export
setMethod("Meta", "GEOData", function(object) {
    return(object@header)
})
#' @export
setMethod("Meta", "GSE", function(object) {
    return(object@header)
})

#' @export
setMethod("Accession", "GEOData", function(object) {
    return(Meta(object)$geo_accession)
})

#' @export
setMethod("dataTable", "GEOData", function(object) {
    return(object@dataTable)
})

#' @export
setMethod("Table", "GEODataTable", function(object) {
    return(object@table)
})

#' @export
setMethod("Columns", "GEODataTable", function(object) {
    return(object@columns)
})

#' @export
setMethod("Table", "GEOData", function(object) {
    return(Table(dataTable(object)))
})

#' @export
setMethod("Columns", "GEOData", function(object) {
    return(Columns(dataTable(object)))
})

#' @export
setMethod("GPLList", "GSE", function(object) {
    return(object@gpls)
})

#' @export
setMethod("GPL", "GDS", function(object) {
    return(object@gpl)
})

#' @export
setMethod("GSMList", "GSE", function(object) {
    return(object@gsms)
})


#' @export
setMethod("show", "GEODataTable", function(object) {
    cat("An object of class \"", class(object), "\"\n", sep = "")
    cat("****** Column Descriptions ******\n")
    print(Columns(object))
    cat("****** Data Table ******\n")
    printHead(Table(object))
})

#' @export
setMethod("show", "GEOData", function(object) {
    cat("An object of class \"", class(object), "\"\n", sep = "")
    for (i in names(Meta(object))) {
        cat(i, "\n")
        print(Meta(object)[[i]])
    }
    print(dataTable(object))
})

---
title: "Using the GEOquery Package"
author: "Sean Davis"
date: "2025-08-16"
last-modified: {{< meta last-modified >}}
format:
  html:
    toc: true
vignette: >
  %\VignetteIndexEntry{Using GEOquery (Quarto)}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---
  
```{r setup}
#| message: false
#| warning: false
#| echo: false
library(GEOquery)
library(knitr)
```

# Introduction to GEO and GEOquery

The NCBI Gene Expression Omnibus ([GEO](https://www.ncbi.nlm.nih.gov/geo/)) was established in 2000 as a public repository for high-throughput molecular abundance data, primarily microarray data at the time. Today, GEO hosts a diverse array of data types including gene expression, genomic DNA, and protein abundance measurements from various technologies like microarrays, next-generation sequencing, and mass spectrometry.

GEOquery was created to bridge the gap between this vast resource and Bioconductor's analytical tools. First released in 2007, GEOquery has evolved alongside GEO itself, adapting to new data types and formats over time.

## Why GEOquery?

Before GEOquery, researchers would need to:

1. Manually download data from the GEO website
2. Parse complex SOFT format files
3. Construct data structures suitable for analysis
4. Integrate metadata with expression data

GEOquery automates this entire process, allowing researchers to focus on analysis rather than data acquisition and formatting.
GEOquery also facilitates automation and reproducibility by incorporating data acquisition into workflows, scripts, or documents.

## GEO Data Organization

Understanding GEO's data organization is essential for effective use of GEOquery:
  
1. **Platform (GPL)**: Describes array design, probes, or detectable elements
2. **Sample (GSM)**: Contains individual experiment measurements
3. **Series (GSE)**: Groups related samples together, typically representing a complete study
4. **Dataset (GDS)**: Curated by GEO staff, represents biologically and statistically comparable samples


# Getting Started with GEOquery

Before working with GEOquery (or any R or Bioconductor package), one must first install the package.
Installation is a one-time (or at least not repeated often) operation. 
To install GEOquery, ensure that you have [installed R and Bioconductor](https://bioconductor.org/install/).

Then, to install GEOquery:

```{r}
#| eval: false
BiocManager::install('GEOquery')
```

Before using GEOquery, we need to load the GEOquery library. 
Loading the GEOquery library must be done _each time you start a new R session_.

```{r}
library(GEOquery)
```

## Downloading a GEO Series

The most common use case is downloading a GEO Series (GSE), which typically represents a complete study:
  
```{r}
# Download GSE2553
gse <- getGEO("GSE2553")
class(gse)
```

Notice that `getGEO` returns a list. This is because a single GSE can contain experiments from multiple platforms. Each element of the list is an `ExpressionSet` containing data from one platform:
  
```{r}
length(gse)
gse[[1]]
```

## Historical Context: SOFT Format vs GSEMatrix Files

GEO originally provided data in SOFT (Simple Omnibus Format in Text) format, which contained extensive information but was slow to parse for large datasets. In response to community needs, GEO introduced GSEMatrix files—a more efficient, tab-delimited format.

GEOquery defaults to using GSEMatrix files (`GSEMatrix=TRUE`) because:
  1. Parsing is substantially faster (often by 10-100x)
2. Memory usage is more efficient
3. The resulting `ExpressionSet` objects are directly usable with Bioconductor tools

# Searching GEO Programmatically

While GEO's web interface is powerful, programmatic searches enable automated data discovery and retrieval. GEOquery provides direct access to GEO's search capabilities:
  
```{r}
#| tbl-cap: "Available GEO search fields."
#| label: tbl-search-fields
# What fields can we search?
fields <- searchFieldsGEO()
kable(fields)
```

GEO uses a specific search syntax with field identifiers in square brackets:
  
```{r}
#| label: tbl-example-search
#| tbl-cap: "Top search results for studies related to COVID-19 in humans with GEO-calculated RNA-seq counts available."
# Find RNA-seq studies related to COVID-19 in humans
results <- searchGEO('covid-19[All Fields] AND "rnaseq counts"[Filter] AND Homo sapiens[ORGN]')
results |>
  dplyr::mutate(Summary=paste(strtrim(Summary,120), '...')) |>
  dplyr::mutate(Title = paste(strtrim(Title, 120), '...')) |>
  head() |>
  kable()
```

The search capabilities mirror GEO's web interface but allow for integration with R workflows.

# RNA-seq Quantifications: NCBI's Solution to Reanalysis Challenges

## The Challenge of RNA-seq Reanalysis

A major barrier to exploiting the massive volume of public RNA-seq data has been the computational cost and expertise required to consistently process raw reads into usable expression values. Different processing pipelines can produce different results, making cross-study comparisons challenging.

## NCBI's RNA-seq Quantification Pipeline

To address this challenge, in 2020-2021, the NCBI SRA and GEO teams developed a standardized pipeline that precomputes RNA-seq gene expression counts for human and mouse datasets. As described in their [documentation](https://www.ncbi.nlm.nih.gov/geo/info/rnaseqcounts.html), this pipeline:
  
  1. Processes RNA-seq data from SRA using the HISAT2 aligner
2. Generates gene expression counts using the featureCounts program
3. Provides consistent annotation based on current genome builds
4. Makes counts available in standardized formats

GEOquery provides direct access to these precomputed counts:
  
```{r}
# Check if RNA-seq quantifications are available
has_quant <- hasRNASeqQuantifications("GSE164073")
has_quant
```

```{r}
#| eval: false
# Get genome build and species information
genome_info <- getRNASeqQuantGenomeInfo("GSE164073")
genome_info
```

```{r}
#| eval: false
# Download and construct a SummarizedExperiment
se <- getRNASeqData("GSE164073")
se
```

This feature saves researchers significant time and computational resources while ensuring standardized processing across datasets.

# Understanding Supplementary Files in GEO

GEO accessions often include supplementary files containing raw data, processing scripts, or additional results not captured in the standard GEO formats. These files are invaluable for:
  
  1. Accessing raw data (e.g., CEL files, FASTQ files)
2. Understanding custom processing pipelines
3. Retrieving additional metadata or results

GEOquery makes accessing these files straightforward:
  
```{r}
# List available supplementary files without downloading
supp_files <- getGEOSuppFiles('GSE63137', fetch_files = FALSE)
head(supp_files)
```

You can filter files by pattern to find specific file types:
  
```{r}
# Find all text files
txt_files <- getGEOSuppFiles('GSE63137', fetch_files = FALSE, 
                             filter_regex = 'txt')
head(txt_files)
```

And download specific files or all supplementary files:
  
```{r}
#| eval: false
# Download all supplementary files for a sample
getGEOSuppFiles('GSM15789') # Files saved to a new directory
```

# Navigating Between R and the GEO Web Interface

Sometimes you may want to examine a GEO record in its web interface. GEOquery provides convenience functions for this:
  
```{r}
#| eval: false
# Get the URL for a GEO accession
url <- urlForAccession("GSE262484")
url

# Open a browser to the GEO page
browseGEOAccession("GSE262484")
```

For RNA-seq datasets specifically, there's a convenience function to search for RNA-seq counts on the GEO website:

```{r}
#| eval: false
browseWebsiteRNASeqSearch()
```

These functions bridge the programmatic and web interfaces to GEO, allowing seamless transitions between analytical and exploratory modes.

# Working with GDS Datasets

GEO DataSets (GDS) are curated collections of samples, processed and normalized to be directly comparable. While less common in modern workflows, they remain available and GEOquery supports them:

```{r}
# Download a GDS dataset
gds <- getGEO("GDS507")
gds
```

GDS objects can be converted to Bioconductor data structures:

```{r}
# Convert to ExpressionSet (with log2 transformation)
eset <- GDS2eSet(gds, do.log2=TRUE)
eset
```

```{r}
# Or to a limma MAList
malist <- GDS2MA(gds)
class(malist)
```

These conversions are particularly useful for integrating older GEO datasets into modern analytical workflows.

# Advanced Features

## Getting GSE Data Tables

Some GSE records contain data tables with important metadata not captured in the standard GSE structure:

```{r}
# Get data tables from GSE3494
dt_list <- getGSEDataTables("GSE3494")
names(dt_list)
head(dt_list[[1]])
```

## Working with GPL Platforms

Platform records (GPL) contain important probe annotations:

```{r}
# Get a platform record
gpl <- getGEO("GPL96")
head(Table(gpl)[, 1:5])
```

When retrieving GSE records, GEOquery can automatically include GPL annotation:

```{r}
#| eval: false
# Get GSE with GPL annotation
gse_with_gpl <- getGEO("GSE2553", AnnotGPL=TRUE)
head(fData(gse_with_gpl[[1]]))
```

# Reporting Bugs and Contributing

As GEO continues to evolve, GEOquery adapts to support new features and data types. If you encounter issues:

1. Check the [Bioconductor Support site](https://support.bioconductor.org/)
2. Report bugs on [GitHub](https://github.com/seandavi/GEOquery/issues)
3. Consider contributing via pull requests

# Citing GEOquery

If you use GEOquery in your research, please cite:

```{r}
citation("GEOquery")
```

# Session Information

```{r}
sessionInfo()
```

---
title: "Handling single-cell RNA-seq data in GEOquery"
author: "Sean Davis"
date: "2025-08-29"
last-modified: {{< meta last-modified >}}
format:
  html:
    toc: true
execute:
  eval: false
vignette: >
  %\VignetteIndexEntry{Handling single-cell RNA-seq data in GEOquery}
  %\VignetteEngine{quarto::html}
  %\VignetteEncoding{UTF-8}
---

```{r message=FALSE, warning=FALSE, eval=TRUE}
library(GEOquery)
library(SingleCellExperiment)
library(DropletUtils)
```

## Single cell searching

```{r}
res = searchGEO("single cell mtx matrix")
head(res)
```


## Mtx files

```{r}
poss= getGEOSuppFiles('GSE248214', fetch_files = FALSE)
poss
```


## Multiple h5ad files

```{r}
library(GEOquery)
s = getGEOSuppFiles('GSE161228', fetch_files = FALSE)
s
```

## Single h5ad file


## SAMPLES

### mtx file

```{r}
s = getGEOSuppFiles('GSM7908437')
s
```
```{r}
sample_prefix = sub('barcodes.tsv.gz','', grep('barcodes.tsv.gz', s$filepath, value = TRUE))
gsm_sce <- DropletUtils::read10xCounts(samples = sample_prefix, type = 'prefix')
gsm_sce
head(colData(gsm_sce))
```

### Tar of mtx files from Series record

```{r}
s = getGEOSuppFiles('GSE248214')
s
```
From here, untar and then wrap to get prefixes

```{r}
tar_filename = rownames(s)[1]
exdir = tempdir()
untar(tar_filename, exdir=exdir)
sample_prefixes = sub('matrix.mtx.gz','',dir(exdir,pattern='*matrix.mtx.gz',full.names = TRUE))
gse_sce = DropletUtils::read10xCounts(samples = sample_prefixes, type='prefix')
gse_sce
```

### Mix of types in a single GSE

```{r}
getGEOSeriesFileListing('GSE288770')
```

### 10x h5 file

```{r}
s = getGEOSuppFiles('GSM8775062')
sce = DropletUtils::read10xCounts(s$filepath)
sce
```

### 10x matrix mtx files

```{r}
s = getGEOSuppFiles('GSM8775066')
sce2 = DropletUtils::read10xCounts(samples = '/Users/davsean/Documents/git/GEOquery/vignettes/GSM8775066/GSM8775066_vivo_day7_1_', type='prefix')
sce2
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEOquery-package.R
\docType{class}
\name{GPL-class}
\alias{GPL-class}
\alias{GPL}
\alias{GPL,GDS-method}
\title{Class 'GPL'}
\description{
Contains a full GEO Platform entity
}
\section{Objects from the Class}{
 Objects can be created by calls of the form
\code{new('GPL', ...)}.
}

\seealso{
\code{\link{GEOData-class}}
}
\author{
Sean Davis
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEOquery-package.R
\name{coercion}
\alias{coercion}
\alias{GDS2MA}
\alias{GDS2eSet}
\title{Convert a GDS data structure to a BioConductor data structure}
\arguments{
\item{GDS}{The GDS datastructure returned by getGEO}

\item{do.log2}{Boolean, should the data in the GDS be log2 transformed
before inserting into the new data structure}

\item{GPL}{Either a GPL data structure (from a call to getGEO) or NULL.  If
NULL, this will cause a call to getGEO to produce a GPL.  The gene
information from the GPL is then used to construct the \code{genes} slot of
the resulting limma \code{MAList} object or the \code{featureData} slot of
the \code{ExpressionSet} instance.}

\item{AnnotGPL}{In general, the annotation GPL files will be available for
GDS records, so the default is to use these files over the user-submitted
GPL files}

\item{getGPL}{A boolean defaulting to TRUE as to whether or not to download
and include GPL information when converting to ExpressionSet or MAList.  You
may want to set this to FALSE if you know that you are going to annotate
your featureData using Bioconductor tools rather than relying on information
provided through NCBI GEO.  Download times can also be greatly reduced by
specifying FALSE.}
}
\value{
\item{GDS2MA}{A limma MAList} \item{GDS2eSet}{An ExpressionSet
object}
}
\description{
Functions to take a GDS data structure from getGEO and coerce it to limma
MALists or ExpressionSets.
}
\details{
This function just rearranges one data structure into another.  For GDS, it
also deals appropriately with making the "targets" list item for the limma
data structure and the phenoData slot of ExpressionSets.
}
\examples{


\dontrun{gds505 <- getGEO('GDS505')}
\dontrun{MA <- GDS2MA(gds505)}
\dontrun{eset <- GDS2eSet(gds505)}


}
\references{
See the limma and ExpressionSet help in the appropriate packages
}
\author{
Sean Davis
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{getRNASeqQuantGenomeInfo}
\alias{getRNASeqQuantGenomeInfo}
\title{Extract genome build and species for GEO RNA-seq quantification}
\usage{
getRNASeqQuantGenomeInfo(gse)
}
\arguments{
\item{gse}{GEO accession number}
}
\value{
A character vector with the genome build and species information
}
\description{
This function extracts the genome build and species information
for a GEO RNA-seq quantification.
}
\examples{
getRNASeqQuantGenomeInfo("GSE164073")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGEOSuppFiles.R
\name{getDirListing}
\alias{getDirListing}
\title{get a directory listing from NCBI GEO}
\usage{
getDirListing(url)
}
\arguments{
\item{url}{A URL, assumed to return an NCBI-formatted
index page}
}
\description{
This one makes some assumptions about the structure of the HTML response
returned.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGEOSuppFiles.R
\name{getGEOSeriesFileListing}
\alias{getGEOSeriesFileListing}
\title{GSE Supplemental file listing}
\usage{
getGEOSeriesFileListing(GSE)
}
\arguments{
\item{GSE}{character(1) the GSE accession}
}
\value{
A data.frame with 5 columns. See example.
}
\description{
The GEO Series records often have one or more supplemental files.
In most cases, those files are archived as '.tar' files, the contents
of which are only available in a file listing file not present on the
website for download.
}
\details{
This function reads that file listing file and returns the results
as a data.frame.
}
\examples{
getGEOSeriesFileListing('GSE288770')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGEOfile.R
\name{getGEOfile}
\alias{getGEOfile}
\title{Download a file from GEO soft file to the local machine}
\usage{
getGEOfile(
  GEO,
  destdir = tempdir(),
  AnnotGPL = FALSE,
  amount = c("full", "brief", "quick", "data")
)
}
\arguments{
\item{GEO}{Character string, the GEO accession for download (eg., GDS84,
GPL96, GSE2553, or GSM10)}

\item{destdir}{Directory in which to store the resulting downloaded file.
Defaults to tempdir()}

\item{AnnotGPL}{A boolean defaulting to FALSE as to whether or not to use
the Annotation GPL information.  These files are nice to use because they
contain up-to-date information remapped from Entrez Gene on a regular basis.
However, they do not exist for all GPLs; in general, they are only available
for GPLs referenced by a GDS}

\item{amount}{Amount of information to pull from GEO.  Only applies to GSE,
GPL, or GSM.  See details...}
}
\value{
Invisibly returns the full path of the downloaded file.
}
\description{
This function simply downloads a SOFT format file associated with the GEO
accession number given.
}
\details{
This function downloads GEO SOFT files based on accession number.  It does
not do any parsing.  The first two arguments should be fairly
self-explanatory, but the last is based on the input to the acc.cgi url at
the geo website.  In the default "full" mode, the entire SOFT format file is
downloaded.  Both "brief" and "quick" offer shortened versions of the files,
good for "peeking" at the file before a big download on a slow connection.
Finally, "data" downloads only the data table part of the SOFT file and is
good for downloading a simple EXCEL-like file for use with other programs (a
convenience).
}
\examples{

  # myfile <- getGEOfile('GDS10')

}
\references{
http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi
}
\seealso{
\code{\link{getGEO}}
}
\author{
Sean Davis
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{getRNAQuantAnnotationURL}
\alias{getRNAQuantAnnotationURL}
\title{Get the RNA-seq quantification annotation link}
\usage{
getRNAQuantAnnotationURL(links)
}
\arguments{
\item{links}{A geoDownloadLinks object}
}
\value{
A character vector with the link to the annotation file
}
\description{
This function extracts the link to the RNA-seq quantification annotation
file from a geoDownloadLinks object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGSEDataTables.R
\name{getGSEDataTables}
\alias{getGSEDataTables}
\title{Get GSE data tables from GEO into R data structures.}
\usage{
getGSEDataTables(GSE)
}
\arguments{
\item{GSE}{The GSE identifier, such as ``GSE3494''.}
}
\value{
A list of data.frames.
}
\description{
In some cases, instead of individual sample records (GSM) containing
information regarding sample phenotypes, the GEO Series contains that
information in an attached data table.  And example is given by GSE3494
where there are two data tables with important information contained within
them.  Using getGEO with the standard parameters downloads the GSEMatrix
file which, unfortunately, does not contain the information in the data
tables.  This function simply downloads the ``header'' information from the
GSE record and parses out the data tables into R data.frames.
}
\examples{

dfl = getGSEDataTables("GSE3494")
lapply(dfl,head)


}
\seealso{
\code{\link{getGEO}}
}
\author{
Sean Davis <sdavis2@mail.nih.gov>
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEOquery-package.R
\docType{class}
\name{GEODataTable-class}
\alias{GEODataTable-class}
\alias{Accession,GEODataTable-method}
\alias{Columns,GEODataTable-method}
\alias{Meta,GEODataTable-method}
\alias{Table,GEODataTable-method}
\alias{dataTable,GEODataTable-method}
\alias{show,GEODataTable-method}
\title{Class "GEODataTable"}
\description{
Contains the column descriptions and data for the datatable part of a GEO
object
}
\section{Objects from the Class}{
 Objects can be created by calls of the form
\code{new("GEODataTable", ...)}.
}

\author{
Sean Davis
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{browseWebsiteRNASeqSearch}
\alias{browseWebsiteRNASeqSearch}
\title{Browse GEO search website for RNA-seq datasets}
\usage{
browseWebsiteRNASeqSearch()
}
\description{
This function opens a browser window to the NCBI GEO website
with a search for RNA-seq datasets. It is included as a convenience
function to remind users of how to search for RNA-seq datasets using
the NCBI GEO website and an "rnaseq counts" filter.
}
\examples{
\dontrun{
browseWebsiteRNASeqSearch()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGEO.R
\name{getGEO}
\alias{getGEO}
\title{Get a GEO object from NCBI or file}
\usage{
getGEO(
  GEO = NULL,
  filename = NULL,
  destdir = tempdir(),
  GSElimits = NULL,
  GSEMatrix = TRUE,
  AnnotGPL = FALSE,
  getGPL = TRUE,
  parseCharacteristics = TRUE
)
}
\arguments{
\item{GEO}{A character string representing a GEO object for download and
parsing.  (eg., 'GDS505','GSE2','GSM2','GPL96')}

\item{filename}{The filename of a previously downloaded GEO SOFT format file
or its gzipped representation (in which case the filename must end in .gz).
Either one of GEO or filename may be specified, not both.  GEO series matrix
files are also handled.  Note that since a single file is being parsed, the
return value is not a list of esets, but a single eset when GSE matrix files
are parsed.}

\item{destdir}{The destination directory for any downloads.  Defaults to the
architecture-dependent tempdir.  You may want to specify a different
directory if you want to save the file for later use.  Doing so is a good
idea if you have a slow connection, as some of the GEO files are HUGE!}

\item{GSElimits}{This argument can be used to load only a contiguous subset
of the GSMs from a GSE.  It should be specified as a vector of length 2
specifying the start and end (inclusive) GSMs to load.  This could be useful
for splitting up large GSEs into more manageable parts, for example.}

\item{GSEMatrix}{A boolean telling GEOquery whether or not to use GSE Series
Matrix files from GEO.  The parsing of these files can be many
orders-of-magnitude faster than parsing the GSE SOFT format files.  Defaults
to TRUE, meaning that the SOFT format parsing will not occur; set to FALSE
if you for some reason need other columns from the GSE records.}

\item{AnnotGPL}{A boolean defaulting to FALSE as to whether or not to use
the Annotation GPL information.  These files are nice to use because they
contain up-to-date information remapped from Entrez Gene on a regular basis.
However, they do not exist for all GPLs; in general, they are only available
for GPLs referenced by a GDS}

\item{getGPL}{A boolean defaulting to TRUE as to whether or not to download
and include GPL information when getting a GSEMatrix file.  You may want to
set this to FALSE if you know that you are going to annotate your
featureData using Bioconductor tools rather than relying on information
provided through NCBI GEO.  Download times can also be greatly reduced by
specifying FALSE.}

\item{parseCharacteristics}{A boolean defaulting to TRUE as to whether or not
to parse the characteristics information (if available) for a GSE Matrix file.
Set this to FALSE if you experience trouble while parsing the characteristics.}
}
\value{
An object of the appropriate class (GDS, GPL, GSM, or GSE) is
returned.  If the GSEMatrix option is used, then a list of ExpressionSet
objects is returned, one for each SeriesMatrix file associated with the GSE
accesion.  If the filename argument is used in combination with a GSEMatrix
file, then the return value is a single ExpressionSet.
}
\description{
This function is the main user-level function in the GEOquery package.  It
directs the download (if no filename is specified) and parsing of a GEO SOFT
format file into an R data structure specifically designed to make access to
each of the important parts of the GEO SOFT format easily accessible.
}
\details{
getGEO functions to download and parse information available from NCBI GEO
(\url{http://www.ncbi.nlm.nih.gov/geo}).  Here are some details about what
is avaible from GEO.  All entity types are handled by getGEO and essentially
any information in the GEO SOFT format is reflected in the resulting data
structure.

From the GEO website:

The Gene Expression Omnibus (GEO) from NCBI serves as a public repository
for a wide range of high-throughput experimental data. These data include
single and dual channel microarray-based experiments measuring mRNA, genomic
DNA, and protein abundance, as well as non-array techniques such as serial
analysis of gene expression (SAGE), and mass spectrometry proteomic data. At
the most basic level of organization of GEO, there are three entity types
that may be supplied by users: Platforms, Samples, and Series.
Additionally, there is a curated entity called a GEO dataset.

A Platform record describes the list of elements on the array (e.g., cDNAs,
oligonucleotide probesets, ORFs, antibodies) or the list of elements that
may be detected and quantified in that experiment (e.g., SAGE tags,
peptides). Each Platform record is assigned a unique and stable GEO
accession number (GPLxxx). A Platform may reference many Samples that have
been submitted by multiple submitters.

A Sample record describes the conditions under which an individual Sample
was handled, the manipulations it underwent, and the abundance measurement
of each element derived from it. Each Sample record is assigned a unique and
stable GEO accession number (GSMxxx). A Sample entity must reference only
one Platform and may be included in multiple Series.

A Series record defines a set of related Samples considered to be part of a
group, how the Samples are related, and if and how they are ordered. A
Series provides a focal point and description of the experiment as a whole.
Series records may also contain tables describing extracted data, summary
conclusions, or analyses. Each Series record is assigned a unique and stable
GEO accession number (GSExxx).

GEO DataSets (GDSxxx) are curated sets of GEO Sample data. A GDS record
represents a collection of biologically and statistically comparable GEO
Samples and forms the basis of GEO's suite of data display and analysis
tools. Samples within a GDS refer to the same Platform, that is, they share
a common set of probe elements. Value measurements for each Sample within a
GDS are assumed to be calculated in an equivalent manner, that is,
considerations such as background processing and normalization are
consistent across the dataset. Information reflecting experimental design is
provided through GDS subsets.
}
\section{Warning }{
 Some of the files that are downloaded, particularly those
associated with GSE entries from GEO are absolutely ENORMOUS and parsing
them can take quite some time and memory.  So, particularly when working
with large GSE entries, expect that you may need a good chunk of memory and
that coffee may be involved when parsing....
}

\examples{

gds <- getGEO("GDS10")
gds

gse <- getGEO('GSE10')
# Returns a list, so look at first item

gse[[1]]

}
\seealso{
\code{\link{getGEOfile}}
}
\author{
Sean Davis
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{getRNASeqQuantResults}
\alias{getRNASeqQuantResults}
\title{Get RNA-seq quantification and annotation from GEO}
\usage{
getRNASeqQuantResults(gse)
}
\arguments{
\item{gse}{GEO accession number}
}
\value{
A list with two elements: quants (a matrix of raw counts) and
annotation (a data frame of annotation information).
}
\description{
This function downloads the raw counts and annotation files from GEO
for a given GEO accession number.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEOquery-package.R
\docType{class}
\name{GEOData-class}
\alias{GEOData-class}
\alias{Accession,GEOData-method}
\alias{Columns,GEOData-method}
\alias{Meta,GEOData-method}
\alias{Table,GEOData-method}
\alias{dataTable,GEOData-method}
\alias{show,GEOData-method}
\title{Class "GEOData"}
\description{
A virtual class for holding GEO samples, platforms, and datasets
}
\section{Objects from the Class}{
 Objects can be created by calls of the form
\code{new("GEOData", ...)}.
}

\seealso{
\code{\link{GDS-class}}, \code{\link{GPL-class}},
\code{\link{GSM-class}}, \code{\link{GEODataTable-class}},
}
\author{
Sean Davis
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGEOfile.R
\name{gunzip}
\alias{gunzip}
\title{Gunzip a file}
\usage{
gunzip(
  filename,
  destname = gsub("[.]gz$", "", filename),
  overwrite = FALSE,
  remove = TRUE,
  BFR.SIZE = 1e+07
)
}
\arguments{
\item{filename}{The filename to be unzipped}

\item{destname}{The destination file}

\item{overwrite}{Boolean indicating whether or not to overwrite a destfile
of the same name}

\item{remove}{Boolean indicating whether or not to remove the original file
after completion}

\item{BFR.SIZE}{The size of the read buffer....}
}
\value{
Invisibly, the number of bytes read.
}
\description{
gunzip a file
}
\details{
This function was stripped out of R.utils due to breaking some stuff on the
bioconductor build machine.
}
\examples{

# gunzip('file.gz',remove=FALSE)


}
\seealso{
\code{\link{gzfile}}
}
\author{
Original author: Henrik Bengtsson
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchGEO.R
\name{searchFieldsGEO}
\alias{searchFieldsGEO}
\title{Provide a list of possible search fields for GEO search}
\usage{
searchFieldsGEO()
}
\value{
a data.frame with names of possible search fields for GEO search
as well as descriptions, data types, etc. for each field. Fields are
in rows and their properties are in columns.
}
\description{
Provide a list of possible search fields for GEO search
}
\examples{
searchFieldsGEO()

}
\seealso{
\code{\link{searchGEO}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEOquery-package.R
\docType{class}
\name{GSM-class}
\alias{GSM-class}
\title{Class "GSM"}
\description{
A class containing a GEO Sample entity
}
\section{Objects from the Class}{
 Objects can be created by calls of the form
\code{new("GSM", ...)}.
}

\seealso{
\code{\link{GEOData-class}}
}
\author{
Sean Davis
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{getRNAQuantRawCountsURL}
\alias{getRNAQuantRawCountsURL}
\title{Get the link to the raw counts file from GEO}
\usage{
getRNAQuantRawCountsURL(links)
}
\arguments{
\item{links}{A geoDownloadLinks object}
}
\value{
A character vector with the link to the raw counts file
}
\description{
This function extracts the link to the raw counts file from a
geoDownloadLinks object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browseGEOAccession.R
\name{browseGEOAccession}
\alias{browseGEOAccession}
\title{Open the GEO page for a given accession}
\usage{
browseGEOAccession(geo)
}
\arguments{
\item{geo}{A GEO accession number}
}
\description{
Sometimes, you just need to see the GEO website page for a
GEO accession. This function opens the GEO page for a given
accession number in the default browser.
}
\examples{
\dontrun{
browseGEOAccession("GSE262484")
}

}
\seealso{
\code{\link{urlForAccession}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGEOSuppFiles.R
\name{getGEOSuppFiles}
\alias{getGEOSuppFiles}
\title{Get Supplemental Files from GEO}
\usage{
getGEOSuppFiles(
  GEO,
  makeDirectory = TRUE,
  baseDir = getwd(),
  fetch_files = TRUE,
  filter_regex = NULL
)
}
\arguments{
\item{GEO}{A GEO accession number such as GPL1073 or GSM1137}

\item{makeDirectory}{Should a "subdirectory" for the downloaded files be
created?  Default is TRUE.  If FALSE, the files will be downloaded directly
into the baseDir.}

\item{baseDir}{The base directory for the downloads.  Default is the current
working directory.}

\item{fetch_files}{logical(1). If TRUE, then actually download the
files. If FALSE, just return the filenames that would have been
downloaded. Useful for testing and getting a list of files
without actual download.}

\item{filter_regex}{A character(1) regular expression that will be
used to filter the filenames from GEO to limit those files that
will be downloaded. This is useful to limit to, for example,
bed files only.}
}
\value{
If fetch_files=TRUE, a data frame is returned invisibly with rownames representing the
full path of the resulting downloaded files and the records in the
data.frame the output of file.info for each downloaded file.
If fetch_files=FALSE, a data.frame of URLs and filenames is returned.
}
\description{
NCBI GEO allows supplemental files to be attached to GEO Series (GSE), GEO
platforms (GPL), and GEO samples (GSM).  This function "knows" how to get
these files based on the GEO accession.  No parsing of the downloaded files
is attempted, since the file format is not generally knowable by the
computer.
}
\details{
Again, just a note that the files are simply downloaded.
}
\examples{

a <- getGEOSuppFiles('GSM1137', fetch_files = FALSE)
a

}
\author{
Sean Davis <sdavis2@mail.nih.gov>
}
\keyword{IO}
\keyword{database}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseGEO.R
\name{parseGEO}
\alias{parseGEO}
\alias{parseGPL}
\alias{parseGSE}
\alias{parseGDS}
\alias{parseGSM}
\title{Parse GEO text}
\usage{
parseGEO(
  fname,
  GSElimits,
  destdir = tempdir(),
  AnnotGPL = FALSE,
  getGPL = TRUE
)
}
\arguments{
\item{fname}{The filename of a SOFT format file.  If the filename ends in
.gz, a gzfile() connection is used to read the file directly.}

\item{GSElimits}{Used to limit the number of GSMs parsed into the GSE
object; useful for memory management for large GSEs.}

\item{destdir}{The destination directory into which files will be saved (to
be used for caching)}

\item{AnnotGPL}{Fetch the annotation GPL if available}

\item{getGPL}{Fetch the GPL associated with a GSEMatrix entity (should
remain TRUE for all normal use cases)}
}
\value{
parseGEO returns an object of the associated type.  For example, if
it is passed the text from a GDS entry, a GDS object is returned.
}
\description{
Workhorse GEO parsers.
}
\details{
These are probably not useful to the end-user.  Use getGEO to access these
functions.  parseGEO simply delegates to the appropriate specific parser.
There should be no reason to use the parseGPL, parseGDS, parseGSE, or
parseGSM functions directly.
}
\seealso{
\code{\link{getGEO}}
}
\author{
Sean Davis
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{getRNASeqData}
\alias{getRNASeqData}
\title{Get GEO RNA-seq quantifications as a SummarizedExperiment object}
\usage{
getRNASeqData(accession)
}
\arguments{
\item{accession}{GEO accession number}
}
\value{
A SummarizedExperiment object with the raw counts as the counts
assay, the annotation as the rowData, and the metadata from GEO as
the colData.
}
\description{
For human and mouse GEO datasets, NCBI GEO attempts to process
the raw data and provide quantifications in the form of raw counts
and an annotation file. This function downloads the raw counts and
annotation files from GEO and merges that with the metadata from the GEO
object to create a SummarizedExperiment.
}
\details{
A major barrier to fully exploiting and reanalyzing the massive volumes
of public RNA-seq data archived by SRA is the cost and effort required to
consistently process raw RNA-seq reads into concise formats that summarize
the expression results. To help address this need, the NCBI SRA and GEO
teams have built a pipeline that precomputes RNA-seq gene expression counts
and delivers them as count matrices that may be incorporated into commonly
used differential expression analysis and visualization software.

The pipeline processes RNA-seq data from SRA using the HISAT2 aligner and
and then generates gene expression counts using the featureCounts program.

See the
\href{https://ncbi.nlm.nih.gov/geo/info/rnaseqcounts.html}{GEO documentation}
for more details.
}
\examples{
se <- getRNASeqData("GSE164073")
se

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browseGEOAccession.R
\name{urlForAccession}
\alias{urlForAccession}
\title{The URL for a GEO accession}
\usage{
urlForAccession(geo)
}
\arguments{
\item{geo}{A GEO accession number}
}
\value{
A character vector with the URL for the GEO accession
}
\description{
Sometimes, you just need the URL for a GEO accession. This function
returns the URL for a given GEO accession number that
can be used to access the GEO page for that accession.
}
\examples{

urlForAccession("GSE262484")

}
\seealso{
\code{\link{browseGEOAccession}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{extractFilenameFromDownloadURL}
\alias{extractFilenameFromDownloadURL}
\title{Extract filename from a GEO download URL}
\usage{
extractFilenameFromDownloadURL(url)
}
\arguments{
\item{url}{A GEO download URL}
}
\value{
A character vector with the filename
}
\description{
This function extracts the filename from a GEO download URL.
The filename is expected to be a query parameter called "file".
If the query parameter is not found, the function returns NULL.
}
\details{
The idea is to use this function to extract filenames that
contain important metadata from the GEO RNA-seq quantification.

In particular, the filename is expected to contain the genome build
and species information that we can attach to the SummarizedExperiment.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGEOSuppFiles.R
\name{getGEOSuppFileURL}
\alias{getGEOSuppFileURL}
\title{Get GEO supplemental file URL for a given GEO accession}
\usage{
getGEOSuppFileURL(GEO)
}
\arguments{
\item{GEO}{}
}
\description{
Get GEO supplemental file URL for a given GEO accession
}
\examples{
# an example of a GEO supplemental file URL
# with a set of single-cell RNA-seq data
url = getGEOSuppFileURL("GSE161228")
url

\dontrun{
  browseURL(url)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parseGEO.R
\name{parseGSEMatrix}
\alias{parseGSEMatrix}
\title{Parse a GSE mstrix file}
\usage{
parseGSEMatrix(
  fname,
  AnnotGPL = FALSE,
  destdir = tempdir(),
  getGPL = TRUE,
  parseCharacteristics = TRUE
)
}
\arguments{
\item{fname}{filename}

\item{AnnotGPL}{set to TRUE to get the annotation GPL version}

\item{destdir}{the destdination directory for download}

\item{getGPL}{whether or not to get the GPL associated}

\item{parseCharacteristics}{Whether or not to do full 'characteristic' parsing}
}
\description{
Not meant for user calling, but parses
a single GSEMatrix file.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEOquery-package.R
\name{GEOData-accessors}
\alias{GEOData-accessors}
\alias{dataTable}
\alias{Accession}
\alias{Columns}
\alias{GPLList}
\alias{GSMList}
\alias{Meta}
\alias{Table}
\title{Generic functions for GEOquery}
\description{
The main documentation is in the Class documentation
}
\seealso{
\code{\link{GEOData-class}}
}
\author{
Sean Davis
}
\keyword{IO}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GEOquery-package.R
\docType{class}
\name{GSE-class}
\alias{GSE-class}
\alias{GPLList,GSE-method}
\alias{GSMList,GSE-method}
\alias{Meta,GSE-method}
\title{Class 'GSE'}
\description{
Contains a GEO Series entity
}
\section{Objects from the Class}{
 Objects can be created by calls of the form
\code{new('GSE', ...)}.
}

\seealso{
\code{\link{GPL-class}},\code{\link{GSM-class}}
}
\author{
Sean Davis
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{getGSEDownloadPageURLs}
\alias{getGSEDownloadPageURLs}
\title{get all download links from a GEO accession}
\usage{
getGSEDownloadPageURLs(gse)
}
\arguments{
\item{gse}{GEO accession number}
}
\value{
A character vector with all download links
}
\description{
This function gets all download links from a GEO accession number.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{readRNAQuantAnnotation}
\alias{readRNAQuantAnnotation}
\title{Read RNA-seq quantification annotation from GEO}
\usage{
readRNAQuantAnnotation(link)
}
\arguments{
\item{link}{A link to the annotation file}
}
\value{
A data frame of annotation information with gene IDs as row names
}
\description{
This function reads the annotation file from a GEO link. The annotation
file is expected to be a tab-separated file with the first column
containing the gene IDs and the remaining columns containing the
annotation information.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{urlExtractRNASeqQuantGenomeInfo}
\alias{urlExtractRNASeqQuantGenomeInfo}
\title{Extract genome build and species from a GEO download URL}
\usage{
urlExtractRNASeqQuantGenomeInfo(url)
}
\arguments{
\item{url}{A GEO annotation file download URL}
}
\value{
A character vector with the genome build and species information
}
\description{
This function extracts the genome build and species information
from a GEO download URL. The genome build and species information
is expected to be in the filename of the download URL.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{hasRNASeqQuantifications}
\alias{hasRNASeqQuantifications}
\title{Does a GEO accession have RNA-seq quantifications?}
\usage{
hasRNASeqQuantifications(accession)
}
\arguments{
\item{accession}{GEO accession number}
}
\value{
TRUE if the GEO accession number has RNA-seq quantifications
available, FALSE otherwise.
}
\description{
This function checks if a GEO accession number has RNA-seq quantifications
available. It does this by checking if the GEO accession number has a
"RNA-Seq raw counts" link available on the GEO download page.
}
\examples{
hasRNASeqQuantifications("GSE164073")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchGEO.R
\name{searchGEO}
\alias{searchGEO}
\title{Search GEO database}
\usage{
searchGEO(query, step = 500L)
}
\arguments{
\item{query}{character, the search term. The NCBI uses a search term syntax
which can be associated with a specific search field with square brackets.
So, for instance "Homo sapiens[ORGN]" denotes a search for \verb{Homo sapiens}
in the “Organism” field. Details see
\url{https://www.ncbi.nlm.nih.gov/geo/info/qqtutorial.html}. The names and
definitions of these fields can be identified using
\link{searchFieldsGEO}.}

\item{step}{the number of records to fetch from the database each time. You
may choose a smaller value if failed.}
}
\value{
a data.frame contains the search results
}
\description{
This function searches the \href{https://www.ncbi.nlm.nih.gov/gds}{GDS}
database, and return a data.frame for all the search results.
}
\details{
The NCBI allows users to access more records (10 per second) if they register
for and use an API key. \link[rentrez:set_entrez_key]{set_entrez_key} function
allows users to set this key for all calls to rentrez functions during a
particular R session. You can also set an environment variable \code{ENTREZ_KEY}
by \link[base:Sys.setenv]{Sys.setenv}.  Once this value is set to your key
rentrez will use it for all requests to the NCBI. Details see
\url{https://docs.ropensci.org/rentrez/articles/rentrez_tutorial.html#rate-limiting-and-api-keys}
}
\examples{
\dontrun{
searchGEO("diabetes[ALL] AND Homo sapiens[ORGN] AND GSE[ETYP]")
}

}
\seealso{
\link{searchFieldsGEO}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnaseq.R
\name{readRNAQuantRawCounts}
\alias{readRNAQuantRawCounts}
\title{Read raw counts from GEO}
\usage{
readRNAQuantRawCounts(link)
}
\arguments{
\item{link}{A link to the raw counts file}
}
\value{
A matrix of raw counts with gene IDs as row names
}
\description{
This function reads the raw counts from a GEO link. The raw counts
are expected to be in a tab-separated file with the first column
containing the gene IDs and the remaining columns containing the
raw counts.
}
\details{
This function reads the raw counts and returns a matrix with the
gene IDs as the row names, ready for use in creating a
SummarizedExperiment.
}
\keyword{internal}
